@@ -23,8 +23,11 @@
 #include "config.h"
 
 #include <netdb.h>
+
 #include "check_data.h"
 #include "check_api.h"
+#include "check_misc.h"
+#include "global_data.h"
 #include "check_ssl.h"
 #include "logger.h"
 #include "memory.h"
@@ -119,7 +122,7 @@ dump_vsg_entry(void *data)
 void
 alloc_vsg(char *gname)
 {
-	int size = strlen(gname);
+	size_t size = strlen(gname);
 	virtual_server_group_t *new;
 
 	new = (virtual_server_group_t *) MALLOC(sizeof(virtual_server_group_t));
@@ -136,19 +139,37 @@ alloc_vsg_entry(vector_t *strvec)
 {
 	virtual_server_group_t *vsg = LIST_TAIL_DATA(check_data->vs_group);
 	virtual_server_group_entry_t *new;
+	uint32_t start;
 
 	new = (virtual_server_group_entry_t *) MALLOC(sizeof(virtual_server_group_entry_t));
 
-	if (!strcmp(vector_slot(strvec, 0), "fwmark")) {
-		new->vfwmark = atoi(vector_slot(strvec, 1));
+	if (!strcmp(strvec_slot(strvec, 0), "fwmark")) {
+		new->vfwmark = (uint32_t)strtoul(strvec_slot(strvec, 1), NULL, 10);
 		list_add(vsg->vfwmark, new);
 	} else {
-		new->range = inet_stor(vector_slot(strvec, 0));
-		inet_stosockaddr(vector_slot(strvec, 0), vector_slot(strvec, 1), &new->addr);
-		if (!new->range)
+		new->range = inet_stor(strvec_slot(strvec, 0));
+		inet_stosockaddr(strvec_slot(strvec, 0), strvec_slot(strvec, 1), &new->addr);
+		if (!new->range) {
 			list_add(vsg->addr_ip, new);
+			return;
+		}
+
+		if ((new->addr.ss_family == AF_INET && new->range > 255 ) ||
+		    (new->addr.ss_family == AF_INET6 && new->range > 0xffff)) {
+			log_message(LOG_INFO, "End address of range exceeds limit for address family - %s - skipping", FMT_STR_VSLOT(strvec, 0));
+			return;
+		}
+
+		if (new->addr.ss_family == AF_INET)
+			start = htonl(((struct sockaddr_in *)&new->addr)->sin_addr.s_addr) & 0xFF;
 		else
-			list_add(vsg->range, new);
+			start = htons(((struct sockaddr_in6 *)&new->addr)->sin6_addr.s6_addr16[7]);
+		if (start >= new->range) {
+			log_message(LOG_INFO, "Address range end is not greater than address range start - %s - skipping", FMT_STR_VSLOT(strvec, 0));
+			return;
+		}
+
+		list_add(vsg->range, new);
 	}
 }
 
@@ -161,8 +182,8 @@ free_vs(void *data)
 	FREE_PTR(vs->virtualhost);
 	FREE_PTR(vs->s_svr);
 	free_list(&vs->rs);
-	FREE_PTR(vs->quorum_up);
-	FREE_PTR(vs->quorum_down);
+	free_notify_script(&vs->quorum_up);
+	free_notify_script(&vs->quorum_down);
 	FREE(vs);
 }
 static void
@@ -185,7 +206,26 @@ dump_vs(void *data)
 	       (vs->delay_loop >= TIMER_MAX_SEC) ? vs->delay_loop/TIMER_HZ :
 						   vs->delay_loop,
 	       vs->sched);
-	log_message(LOG_INFO, "   One packet scheduling = %sabled%s", vs->ops ? "en" : "dis", (vs->ops && vs->service_type != IPPROTO_UDP) ? " (inactive due to not UDP)" : "");
+	log_message(LOG_INFO, "   Hashed = %sabled", vs->flags & IP_VS_SVC_F_HASHED ? "en" : "dis");
+#ifdef IP_VS_SVC_F_SCHED1
+	if (!strcmp(vs->sched, "sh"))
+	{
+		log_message(LOG_INFO, "   sh-port = %sabled", vs->flags & IP_VS_SVC_F_SCHED_SH_PORT ? "en" : "dis");
+		log_message(LOG_INFO, "   sh-fallback = %sabled", vs->flags & IP_VS_SVC_F_SCHED_SH_FALLBACK ? "en" : "dis");
+	}
+	else
+	{
+		log_message(LOG_INFO, "   flag-1 = %sabled", vs->flags & IP_VS_SVC_F_SCHED1 ? "en" : "dis");
+		log_message(LOG_INFO, "   flag-2 = %sabled", vs->flags & IP_VS_SVC_F_SCHED2 ? "en" : "dis");
+		log_message(LOG_INFO, "   flag-3 = %sabled", vs->flags & IP_VS_SVC_F_SCHED3 ? "en" : "dis");
+	}
+#endif
+#ifdef IP_VS_SVC_F_ONEPACKET
+	log_message(LOG_INFO, "   One packet scheduling = %sabled%s",
+			(vs->flags & IP_VS_SVC_F_ONEPACKET) ? "en" : "dis",
+			((vs->flags & IP_VS_SVC_F_ONEPACKET) && vs->service_type != IPPROTO_UDP) ? " (inactive due to not UDP)" : "");
+#endif
+
 	if (vs->persistence_timeout)
 		log_message(LOG_INFO, "   persistence timeout = %u", vs->persistence_timeout);
 	if (vs->persistence_granularity) {
@@ -206,13 +246,13 @@ dump_vs(void *data)
 		log_message(LOG_INFO, "   protocol = %d", vs->service_type);
 	log_message(LOG_INFO, "   alpha is %s, omega is %s",
 		    vs->alpha ? "ON" : "OFF", vs->omega ? "ON" : "OFF");
-	log_message(LOG_INFO, "   quorum = %lu, hysteresis = %lu", vs->quorum, vs->hysteresis);
+	log_message(LOG_INFO, "   quorum = %u, hysteresis = %u", vs->quorum, vs->hysteresis);
 	if (vs->quorum_up)
-		log_message(LOG_INFO, "   -> Notify script UP = %s",
-			    vs->quorum_up);
+		log_message(LOG_INFO, "   -> Notify script UP = %s, uid:gid %d:%d",
+			    vs->quorum_up->name, vs->quorum_up->uid, vs->quorum_up->gid);
 	if (vs->quorum_down)
-		log_message(LOG_INFO, "   -> Notify script DOWN = %s",
-			    vs->quorum_down);
+		log_message(LOG_INFO, "   -> Notify script DOWN = %s, uid:gid %d:%d",
+			    vs->quorum_down->name, vs->quorum_down->uid, vs->quorum_down->gid);
 	if (vs->ha_suspend)
 		log_message(LOG_INFO, "   Using HA suspend");
 
@@ -241,7 +281,7 @@ dump_vs(void *data)
 void
 alloc_vs(char *ip, char *port)
 {
-	int size = strlen(port);
+	size_t size = strlen(port);
 	virtual_server_t *new;
 
 	new = (virtual_server_t *) MALLOC(sizeof(virtual_server_t));
@@ -250,7 +290,7 @@ alloc_vs(char *ip, char *port)
 		new->vsgname = (char *) MALLOC(size + 1);
 		memcpy(new->vsgname, port, size);
 	} else if (!strcmp(ip, "fwmark")) {
-		new->vfwmark = atoi(port);
+		new->vfwmark = (uint32_t)strtoul(port, NULL, 10);
 	} else {
 		inet_stosockaddr(ip, port, &new->addr);
 		new->af = new->addr.ss_family;
@@ -258,13 +298,14 @@ alloc_vs(char *ip, char *port)
 
 	new->delay_loop = KEEPALIVED_DEFAULT_DELAY;
 	new->virtualhost = NULL;
-	new->alpha = 0;
-	new->omega = 0;
+	new->alpha = false;
+	new->omega = false;
 	new->quorum_up = NULL;
 	new->quorum_down = NULL;
 	new->quorum = 1;
 	new->hysteresis = 0;
 	new->quorum_state = UP;
+	new->flags = 0;
 
 	list_add(check_data->vs, new);
 }
@@ -289,8 +330,8 @@ static void
 free_rs(void *data)
 {
 	real_server_t *rs = data;
-	FREE_PTR(rs->notify_up);
-	FREE_PTR(rs->notify_down);
+	free_notify_script(&rs->notify_up);
+	free_notify_script(&rs->notify_down);
 	free_list(&rs->failed_checkers);
 	FREE(rs);
 }
@@ -306,11 +347,11 @@ dump_rs(void *data)
 	if (rs->inhibit)
 		log_message(LOG_INFO, "     -> Inhibit service on failure");
 	if (rs->notify_up)
-		log_message(LOG_INFO, "     -> Notify script UP = %s",
-		       rs->notify_up);
+		log_message(LOG_INFO, "     -> Notify script UP = %s, uid:gid %d:%d",
+		       rs->notify_up->name, rs->notify_up->uid, rs->notify_up->gid);
 	if (rs->notify_down)
-		log_message(LOG_INFO, "     -> Notify script DOWN = %s",
-		       rs->notify_down);
+		log_message(LOG_INFO, "     -> Notify script DOWN = %s, uid:gid %d:%d",
+		       rs->notify_down->name, rs->notify_down->uid, rs->notify_down->gid);
 }
 
 static void
@@ -397,3 +438,59 @@ format_vs (virtual_server_t *vs)
 
 	return ret;
 }
+
+static void
+check_check_script_security(void)
+{
+	element e, e1;
+	virtual_server_t *vs;
+	real_server_t *rs;
+	int script_flags;
+
+	if (LIST_ISEMPTY(check_data->vs))
+		return;
+
+	script_flags = check_misc_script_security();
+
+	for (e = LIST_HEAD(check_data->vs); e; ELEMENT_NEXT(e)) {
+		vs = ELEMENT_DATA(e);
+
+		script_flags |= check_notify_script_secure(&vs->quorum_up, global_data->script_security, false);
+		script_flags |= check_notify_script_secure(&vs->quorum_down, global_data->script_security, false);
+
+		for (e1 = LIST_HEAD(vs->rs); e1; ELEMENT_NEXT(e1)) {
+			rs = ELEMENT_DATA(e1);
+
+			script_flags |= check_notify_script_secure(&rs->notify_up, global_data->script_security, false);
+			script_flags |= check_notify_script_secure(&rs->notify_down, global_data->script_security, false);
+		}
+	}
+
+	if (!global_data->script_security && script_flags & SC_ISSCRIPT) {
+		log_message(LOG_INFO, "SECURITY VIOLATION - check scripts are being executed but script_security not enabled.%s",
+				script_flags & SC_INSECURE ? " There are insecure scripts." : "");
+	}
+}
+
+bool validate_check_config(void)
+{
+	element e;
+	virtual_server_t *vs;
+
+	/* Ensure that no virtual server hysteresis >= quorum */
+	if (!LIST_ISEMPTY(check_data->vs)) {
+		for (e = LIST_HEAD(check_data->vs); e; ELEMENT_NEXT(e)) {
+			vs = ELEMENT_DATA(e);
+
+			if (vs->hysteresis >= vs->quorum) {
+				log_message(LOG_INFO, "Virtual server %s: hysteresis %u >= quorum %u; setting hysteresis to %u",
+						vs->vsgname, vs->hysteresis, vs->quorum, vs->quorum -1);
+				vs->hysteresis = vs->quorum - 1;
+			}
+		}
+	}
+
+	check_check_script_security();
+
+	return true;
+}
