@@ -32,6 +32,7 @@
 #ifdef _WITH_SNMP_CHECKER_
   #include "check_snmp.h"
 #endif
+#include "global_data.h"
 
 /* out-of-order functions declarations */
 static void update_quorum_state(virtual_server_t * vs);
@@ -80,7 +81,7 @@ clear_service_rs(virtual_server_t * vs, list l)
 			 */
 			if (rs->notify_down) {
 				log_message(LOG_INFO, "Executing [%s] for service %s in VS %s"
-						    , rs->notify_down
+						    , rs->notify_down->name
 						    , FMT_RS(rs)
 						    , FMT_VS(vs));
 				notify_exec(rs->notify_down);
@@ -101,7 +102,7 @@ clear_service_rs(virtual_server_t * vs, list l)
 				vs->quorum_state = DOWN;
 				if (vs->quorum_down) {
 					log_message(LOG_INFO, "Executing [%s] for VS %s"
-							    , vs->quorum_down
+							    , vs->quorum_down->name
 							    , FMT_VS(vs));
 					notify_exec(vs->quorum_down);
 				}
@@ -116,7 +117,7 @@ clear_service_rs(virtual_server_t * vs, list l)
 }
 
 /* Remove a virtualserver IPVS rule */
-static int
+static bool
 clear_service_vs(virtual_server_t * vs)
 {
 	/* Processing real server queue */
@@ -125,18 +126,18 @@ clear_service_vs(virtual_server_t * vs)
 			if (ISALIVE(vs->s_svr))
 				ipvs_cmd(LVS_CMD_DEL_DEST, vs, vs->s_svr);
 		} else if (!clear_service_rs(vs, vs->rs))
-			return 0;
+			return false;
 		/* The above will handle Omega case for VS as well. */
 	}
 
 	ipvs_cmd(LVS_CMD_DEL, vs, NULL);
 
 	UNSET_ALIVE(vs);
-	return 1;
+	return true;
 }
 
 /* IPVS cleaner processing */
-int
+void
 clear_services(void)
 {
 	element e;
@@ -146,22 +147,20 @@ clear_services(void)
 	for (e = LIST_HEAD(l); e; ELEMENT_NEXT(e)) {
 		vs = ELEMENT_DATA(e);
 		if (!clear_service_vs(vs))
-			return 0;
+			return;
 	}
-	return 1;
 }
 
 /* Set a realserver IPVS rules */
-static int
+static bool
 init_service_rs(virtual_server_t * vs)
 {
 	element e;
 	real_server_t *rs;
 
 	if (LIST_ISEMPTY(vs->rs)) {
-		log_message(LOG_WARNING, "VS [%s] has no configured RS! Skipping RS activation."
-				       , FMT_VS(vs));
-		return 1;
+		log_message(LOG_WARNING, "VS [%s] has no configured RS! Skipping RS activation.", FMT_VS(vs));
+		return true;
 	}
 
 	for (e = LIST_HEAD(vs->rs); e; ELEMENT_NEXT(e)) {
@@ -169,7 +168,7 @@ init_service_rs(virtual_server_t * vs)
 
 		if (rs->reloaded) {
 			if (rs->iweight != rs->pweight)
-				update_svr_wgt(rs->iweight, vs, rs, 0);
+				update_svr_wgt(rs->iweight, vs, rs, false);
 			/* Do not re-add failed RS instantly on reload */
 			continue;
 		}
@@ -183,7 +182,7 @@ init_service_rs(virtual_server_t * vs)
 		}
 	}
 
-	return 1;
+	return true;
 }
 
 static void
@@ -219,7 +218,7 @@ sync_service_vsg(virtual_server_t * vs)
 }
 
 /* Set a virtualserver IPVS rules */
-static int
+static bool
 init_service_vs(virtual_server_t * vs)
 {
 	/* Init the VS root */
@@ -230,7 +229,7 @@ init_service_vs(virtual_server_t * vs)
 
 	/* Processing real server queue */
 	if (!init_service_rs(vs))
-		return 0;
+		return false;
 
 	if (vs->reloaded) {
 		if (vs->vsgname)
@@ -241,11 +240,11 @@ init_service_vs(virtual_server_t * vs)
 		update_quorum_state(vs);
 	}
 
-	return 1;
+	return true;
 }
 
 /* Set IPVS rules */
-int
+bool
 init_services(void)
 {
 	element e;
@@ -255,14 +254,14 @@ init_services(void)
 	for (e = LIST_HEAD(l); e; ELEMENT_NEXT(e)) {
 		vs = ELEMENT_DATA(e);
 		if (!init_service_vs(vs))
-			return 0;
+			return false;
 	}
-	return 1;
+	return true;
 }
 
 /* add or remove _alive_ real servers from a virtual server */
 static void
-perform_quorum_state(virtual_server_t *vs, int add)
+perform_quorum_state(virtual_server_t *vs, bool add)
 {
 	element e;
 	real_server_t *rs;
@@ -278,9 +277,9 @@ perform_quorum_state(virtual_server_t *vs, int add)
 		if (!ISALIVE(rs)) /* We only handle alive servers */
 			continue;
 		if (add)
-			rs->alive = 0;
+			rs->alive = false;
 		ipvs_cmd(add?LVS_CMD_ADD_DEST:LVS_CMD_DEL_DEST, vs, rs);
-		rs->alive = 1;
+		rs->alive = true;
 	}
 }
 
@@ -296,7 +295,7 @@ update_quorum_state(virtual_server_t * vs)
 	if (vs->quorum_state == DOWN &&
 	    weight_sum >= up_threshold) {
 		vs->quorum_state = UP;
-		log_message(LOG_INFO, "Gained quorum %lu+%lu=%li <= %li for VS %s"
+		log_message(LOG_INFO, "Gained quorum %u+%u=%ld <= %ld for VS %s"
 				    , vs->quorum
 				    , vs->hysteresis
 				    , up_threshold
@@ -309,19 +308,19 @@ update_quorum_state(virtual_server_t * vs)
 					    , FMT_VS(vs));
 
 			ipvs_cmd(LVS_CMD_DEL_DEST, vs, vs->s_svr);
-			vs->s_svr->alive = 0;
+			vs->s_svr->alive = false;
 
 			/* Adding back alive real servers */
-			perform_quorum_state(vs, 1);
+			perform_quorum_state(vs, true);
 		}
 		if (vs->quorum_up) {
 			log_message(LOG_INFO, "Executing [%s] for VS %s"
-					    , vs->quorum_up
+					    , vs->quorum_up->name
 					    , FMT_VS(vs));
 			notify_exec(vs->quorum_up);
 		}
 #ifdef _WITH_SNMP_CHECKER_
-	       check_snmp_quorum_trap(vs);
+		check_snmp_quorum_trap(vs);
 #endif
 		return;
 	}
@@ -333,7 +332,7 @@ update_quorum_state(virtual_server_t * vs)
 	    (!weight_sum || weight_sum < down_threshold)
 	) {
 		vs->quorum_state = DOWN;
-		log_message(LOG_INFO, "Lost quorum %lu-%lu=%li > %li for VS %s"
+		log_message(LOG_INFO, "Lost quorum %u-%u=%ld > %ld for VS %s"
 				    , vs->quorum
 				    , vs->hysteresis
 				    , down_threshold
@@ -341,7 +340,7 @@ update_quorum_state(virtual_server_t * vs)
 				    , FMT_VS(vs));
 		if (vs->quorum_down) {
 			log_message(LOG_INFO, "Executing [%s] for VS %s"
-					    , vs->quorum_down
+					    , vs->quorum_down->name
 					    , FMT_VS(vs));
 			notify_exec(vs->quorum_down);
 		}
@@ -353,10 +352,10 @@ update_quorum_state(virtual_server_t * vs)
 
 			/* the sorry server is now up in the pool, we flag it alive */
 			ipvs_cmd(LVS_CMD_ADD_DEST, vs, vs->s_svr);
-			vs->s_svr->alive = 1;
+			vs->s_svr->alive = true;
 
 			/* Remove remaining alive real servers */
-			perform_quorum_state(vs, 0);
+			perform_quorum_state(vs, false);
 		}
 #ifdef _WITH_SNMP_CHECKER_
 		check_snmp_quorum_trap(vs);
@@ -367,14 +366,14 @@ update_quorum_state(virtual_server_t * vs)
 
 /* manipulate add/remove rs according to alive state */
 static int
-perform_svr_state(int alive, virtual_server_t * vs, real_server_t * rs)
+perform_svr_state(bool alive, virtual_server_t * vs, real_server_t * rs)
 {
 	/*
 	 * | ISALIVE(rs) | alive | context
-	 * | 0           | 0     | first check failed under alpha mode, unreachable here
-	 * | 0           | 1     | RS went up, add it to the pool
-	 * | 1           | 0     | RS went down, remove it from the pool
-	 * | 1           | 1     | first check succeeded w/o alpha mode, unreachable here
+	 * | false       | false | first check failed under alpha mode, unreachable here
+	 * | false       | true  | RS went up, add it to the pool
+	 * | true        | false | RS went down, remove it from the pool
+	 * | true        | true  | first check succeeded w/o alpha mode, unreachable here
 	 */
 	if (!ISALIVE(rs) && alive) {
 		log_message(LOG_INFO, "%s service %s to VS %s"
@@ -389,7 +388,7 @@ perform_svr_state(int alive, virtual_server_t * vs, real_server_t * rs)
 		rs->alive = alive;
 		if (rs->notify_up) {
 			log_message(LOG_INFO, "Executing [%s] for service %s in VS %s"
-					    , rs->notify_up
+					    , rs->notify_up->name
 					    , FMT_RS(rs)
 					    , FMT_VS(vs));
 			notify_exec(rs->notify_up);
@@ -418,7 +417,7 @@ perform_svr_state(int alive, virtual_server_t * vs, real_server_t * rs)
 		rs->alive = alive;
 		if (rs->notify_down) {
 			log_message(LOG_INFO, "Executing [%s] for service %s in VS %s"
-					    , rs->notify_down
+					    , rs->notify_down->name
 					    , FMT_RS(rs)
 					    , FMT_VS(vs));
 			notify_exec(rs->notify_down);
@@ -436,7 +435,7 @@ perform_svr_state(int alive, virtual_server_t * vs, real_server_t * rs)
 /* Store new weight in real_server struct and then update kernel. */
 void
 update_svr_wgt(int weight, virtual_server_t * vs, real_server_t * rs
-		, int update_quorum)
+		, bool update_quorum)
 {
 	if (weight != rs->weight) {
 		log_message(LOG_INFO, "Changing weight from %d to %d for %s service %s of VS %s"
@@ -484,7 +483,7 @@ svr_checker_up(checker_id_t cid, real_server_t *rs)
 
 /* Update checker's state */
 void
-update_svr_checker_state(int alive, checker_id_t cid, virtual_server_t *vs, real_server_t *rs)
+update_svr_checker_state(bool alive, checker_id_t cid, virtual_server_t *vs, real_server_t *rs)
 {
 	element e;
 	list l = rs->failed_checkers;
@@ -548,7 +547,7 @@ vsge_exist(virtual_server_group_entry_t *vsg_entry, list l)
 }
 
 /* Clear the diff vsge of old group */
-static int
+static void
 clear_diff_vsge(list old, list new, virtual_server_t * old_vs)
 {
 	virtual_server_group_entry_t *vsge, *new_vsge;
@@ -559,7 +558,7 @@ clear_diff_vsge(list old, list new, virtual_server_t * old_vs)
 		new_vsge = vsge_exist(vsge, new);
 		if (new_vsge) {
 			new_vsge->alive = vsge->alive;
-			new_vsge->reloaded = 1;
+			new_vsge->reloaded = true;
 		}
 		else {
 			log_message(LOG_INFO, "VS [%s:%d:%u] in group %s no longer exist"
@@ -571,26 +570,19 @@ clear_diff_vsge(list old, list new, virtual_server_t * old_vs)
 			ipvs_group_remove_entry(old_vs, vsge);
 		}
 	}
-
-	return 1;
 }
 
 /* Clear the diff vsg of the old vs */
-static int
+static void
 clear_diff_vsg(virtual_server_t * old_vs, virtual_server_t * new_vs)
 {
 	virtual_server_group_t *old = old_vs->vsg;
 	virtual_server_group_t *new = new_vs->vsg;
 
 	/* Diff the group entries */
-	if (!clear_diff_vsge(old->addr_ip, new->addr_ip, old_vs))
-		return 0;
-	if (!clear_diff_vsge(old->range, new->range, old_vs))
-		return 0;
-	if (!clear_diff_vsge(old->vfwmark, new->vfwmark, old_vs))
-		return 0;
-
-	return 1;
+	clear_diff_vsge(old->addr_ip, new->addr_ip, old_vs);
+	clear_diff_vsge(old->range, new->range, old_vs);
+	clear_diff_vsge(old->vfwmark, new->vfwmark, old_vs);
 }
 
 /* Check if a vs exist in new data and returns pointer to it */
@@ -665,7 +657,7 @@ clear_diff_rs(virtual_server_t * old_vs, list new_rs_list)
 			new_rs->set = rs->set;
 			new_rs->weight = rs->weight;
 			new_rs->pweight = rs->iweight;
-			new_rs->reloaded = 1;
+			new_rs->reloaded = true;
 			if (new_rs->alive) {
 				/* clear failed_checkers list */
 				free_list_elements(new_rs->failed_checkers);
@@ -696,7 +688,7 @@ clear_diff_rs(virtual_server_t * old_vs, list new_rs_list)
 
 /* When reloading configuration, remove negative diff entries
  * and copy status of existing entries to the new ones */
-int
+void
 clear_diff_services(void)
 {
 	element e;
@@ -705,7 +697,7 @@ clear_diff_services(void)
 
 	/* If old config didn't own vs then nothing return */
 	if (LIST_ISEMPTY(l))
-		return 1;
+		return;
 
 	/* Remove diff entries from previous IPVS rules */
 	for (e = LIST_HEAD(l); e; ELEMENT_NEXT(e)) {
@@ -718,38 +710,32 @@ clear_diff_services(void)
 		new_vs = vs_exist(vs);
 		if (!new_vs) {
 			if (vs->vsgname)
-				log_message(LOG_INFO, "Removing Virtual Server Group [%s]"
-						    , vs->vsgname);
+				log_message(LOG_INFO, "Removing Virtual Server Group [%s]", vs->vsgname);
 			else
-				log_message(LOG_INFO, "Removing Virtual Server %s"
-						    , FMT_VS(vs));
+				log_message(LOG_INFO, "Removing Virtual Server %s", FMT_VS(vs));
 
 			/* Clear VS entry */
 			if (!clear_service_vs(vs))
-				return 0;
+				return;
 		} else {
 			/* copy status fields from old VS */
 			SET_ALIVE(new_vs);
 			new_vs->quorum_state = vs->quorum_state;
-			new_vs->reloaded = 1;
+			new_vs->reloaded = true;
 
 			if (vs->vsgname)
 				clear_diff_vsg(vs, new_vs);
 
 			/* If vs exist, perform rs pool diff */
-			/* omega = 0 must not prevent the notifiers from being called,
+			/* omega = false must not prevent the notifiers from being called,
 			   because the VS still exists in new configuration */
-			vs->omega = 1;
+			vs->omega = true;
 			if (!clear_diff_rs(vs, new_vs->rs))
-				return 0;
+				return;
 			if (vs->s_svr && ISALIVE(vs->s_svr))
-				ipvs_cmd(LVS_CMD_DEL_DEST
-					      , vs
-					      , vs->s_svr);
+				ipvs_cmd(LVS_CMD_DEL_DEST, vs, vs->s_svr);
 		}
 	}
-
-	return 1;
 }
 
 void
