@@ -1,19 +1,19 @@
 /*
  * libipvs:	Library for manipulating IPVS through netlink or [gs]etsockopt
  *
- * 		This code is copied from the ipvsadm sources, with the unused
- * 		code removed. It is available at:
- * 		https://git.kernel.org/cgit/utils/kernel/ipvsadm/ipvsadm.git
+ *		This code is copied from the ipvsadm sources, with the unused
+ *		code removed. It is available at:
+ *		https://git.kernel.org/cgit/utils/kernel/ipvsadm/ipvsadm.git
  *
- * 		The upstream code should periodically be checked for updates,
- * 		which should then be applied to this code.
+ *		The upstream code should periodically be checked for updates,
+ *		which should then be applied to this code.
  *
- * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
+ * Authors:	Wensong Zhang <wensong@linuxvirtualserver.org>
  *
- *              This program is free software; you can redistribute it and/or
- *              modify it under the terms of the GNU General Public License
- *              as published by the Free Software Foundation; either version
- *              2 of the License, or (at your option) any later version.
+ *		This program is free software; you can redistribute it and/or
+ *		modify it under the terms of the GNU General Public License
+ *		as published by the Free Software Foundation; either version
+ *		2 of the License, or (at your option) any later version.
  */
 
 #include "config.h"
@@ -83,11 +83,11 @@ static struct nla_policy ipvs_service_policy[IPVS_SVC_ATTR_MAX + 1] = {
 	[IPVS_SVC_ATTR_TIMEOUT]		= { .type = NLA_U32 },
 	[IPVS_SVC_ATTR_NETMASK]		= { .type = NLA_U32 },
 	[IPVS_SVC_ATTR_STATS]		= { .type = NLA_NESTED },
-#ifdef IPVS_SVC_ATTR_PE_NAME		/* Since Linux 2.6.37 */
+#ifdef _HAVE_PE_NAME_
 	[IPVS_SVC_ATTR_PE_NAME]		= { .type = NLA_STRING,
-					    .maxlen = IP_VS_PENAME_MAXLEN }
+					    .maxlen = IP_VS_PENAME_MAXLEN },
 #endif
-#ifdef IPVS_SVC_ATTR_STATS64		/* Since Linux 4.0 */
+#ifdef _WITH_LVS_64BIT_STATS_
 	[IPVS_SVC_ATTR_STATS64]		= { .type = NLA_NESTED },
 #endif
 };
@@ -104,10 +104,10 @@ static struct nla_policy ipvs_dest_policy[IPVS_DEST_ATTR_MAX + 1] = {
 	[IPVS_DEST_ATTR_INACT_CONNS]	= { .type = NLA_U32 },
 	[IPVS_DEST_ATTR_PERSIST_CONNS]	= { .type = NLA_U32 },
 	[IPVS_DEST_ATTR_STATS]		= { .type = NLA_NESTED },
-#ifdef IPVS_DEST_ATTR_ADDR_FAMILY	/* Since Linux 3.18 */
+#if HAVE_DECL_IPVS_DEST_ATTR_ADDR_FAMILY
 	[IPVS_DEST_ATTR_ADDR_FAMILY]	= { .type = NLA_U16 },
 #endif
-#ifdef IPVS_DEST_ATTR_STATS64		/* Since Linux 4.0 */
+#ifdef _WITH_LVS_64BIT_STATS_
 	[IPVS_DEST_ATTR_STATS64]	= {.type = NLA_NESTED },
 #endif
 };
@@ -146,14 +146,14 @@ static struct nla_policy ipvs_stats_policy[IPVS_STATS_ATTR_MAX + 1] = {
 	{ errno = EAFNOSUPPORT; goto out_err; }			\
 	s->user.addr = s->nf_addr.ip;				\
 
-#ifdef IPVS_SVC_ATTR_PE_NAME
+#ifdef _HAVE_PE_NAME_
 #define CHECK_PE(s, ret) if (s->pe_name[0])			\
 	{ errno = EAFNOSUPPORT; goto out_err; }
 #endif
 
 #define CHECK_COMPAT_DEST(s, ret) CHECK_IPV4(s, ret)
 
-#ifdef IPVS_SVC_ATTR_PE_NAME
+#ifdef _HAVE_PE_NAME_
 #define CHECK_COMPAT_SVC(s, ret)				\
 	CHECK_IPV4(s, ret);					\
 	CHECK_PE(s, ret);
@@ -162,6 +162,17 @@ static struct nla_policy ipvs_stats_policy[IPVS_STATS_ATTR_MAX + 1] = {
 	CHECK_IPV4(s, ret);
 #endif
 
+#ifndef NLA_PUT_S32
+#define NLA_PUT_S32(msg, attrtype, value) \
+	NLA_PUT_TYPE(msg, int32_t, attrtype, value)
+
+static inline int32_t
+nla_get_s32(struct nlattr *attr)
+{
+	return (int32_t)nla_get_u32(attr);
+}
+#endif
+
 #ifdef LIBIPVS_USE_NL
 #ifndef FALLBACK_LIBNL1
 static int nlerr2syserr(int err)
@@ -188,7 +199,7 @@ static int nlerr2syserr(int err)
 }
 #endif
 
-static struct nl_msg *ipvs_nl_message(int cmd, int flags)
+static struct nl_msg *ipvs_nl_message(uint8_t cmd, int flags)
 {
 	struct nl_msg *msg;
 
@@ -202,7 +213,7 @@ static struct nl_msg *ipvs_nl_message(int cmd, int flags)
 	return msg;
 }
 
-static int ipvs_nl_noop_cb(struct nl_msg *msg, void *arg)
+static int ipvs_nl_noop_cb(__attribute__((unused)) struct nl_msg *msg, __attribute__((unused)) void *arg)
 {
 	return NL_OK;
 }
@@ -303,7 +314,7 @@ static int ipvs_nl_fill_service_attr(struct nl_msg *msg, ipvs_service_t *svc)
 {
 	struct nlattr *nl_service;
 	struct ip_vs_flags flags = { .flags = svc->user.flags,
-				     .mask = ~0 };
+				     .mask = ~0U };
 
 	nl_service = nla_nest_start(msg, IPVS_CMD_ATTR_SERVICE);
 	if (!nl_service)
@@ -320,7 +331,7 @@ static int ipvs_nl_fill_service_attr(struct nl_msg *msg, ipvs_service_t *svc)
 	}
 
 	NLA_PUT_STRING(msg, IPVS_SVC_ATTR_SCHED_NAME, svc->user.sched_name);
-#ifdef IPVS_SVC_ATTR_PE_NAME
+#ifdef _HAVE_PE_NAME_
 	if (svc->pe_name[0])
 		NLA_PUT_STRING(msg, IPVS_SVC_ATTR_PE_NAME, svc->pe_name);
 #endif
@@ -438,13 +449,13 @@ static int ipvs_nl_fill_dest_attr(struct nl_msg *msg, ipvs_dest_t *dst)
 	if (!nl_dest)
 		return -1;
 
-#ifdef IPVS_DEST_ATTR_ADDR_FAMILY
+#if HAVE_DECL_IPVS_DEST_ATTR_ADDR_FAMILY
 	NLA_PUT_U16(msg, IPVS_DEST_ATTR_ADDR_FAMILY, dst->af);
 #endif
 	NLA_PUT(msg, IPVS_DEST_ATTR_ADDR, sizeof(dst->nf_addr), &(dst->nf_addr));
 	NLA_PUT_U16(msg, IPVS_DEST_ATTR_PORT, dst->user.port);
 	NLA_PUT_U32(msg, IPVS_DEST_ATTR_FWD_METHOD, dst->user.conn_flags & IP_VS_CONN_F_FWD_MASK);
-	NLA_PUT_U32(msg, IPVS_DEST_ATTR_WEIGHT, dst->user.weight);
+	NLA_PUT_U32(msg, IPVS_DEST_ATTR_WEIGHT, (uint32_t)dst->user.weight);
 	NLA_PUT_U32(msg, IPVS_DEST_ATTR_U_THRESH, dst->user.u_threshold);
 	NLA_PUT_U32(msg, IPVS_DEST_ATTR_L_THRESH, dst->user.l_threshold);
 
@@ -561,11 +572,11 @@ int ipvs_set_timeout(ipvs_timeout_t *to)
 		if (!msg) return -1;
 
 		if (to->tcp_timeout)
-			NLA_PUT_U32(msg, IPVS_CMD_ATTR_TIMEOUT_TCP, to->tcp_timeout);
+			NLA_PUT_U32(msg, IPVS_CMD_ATTR_TIMEOUT_TCP, (uint32_t)to->tcp_timeout);
 		if (to->tcp_fin_timeout)
-			NLA_PUT_U32(msg, IPVS_CMD_ATTR_TIMEOUT_TCP_FIN, to->tcp_fin_timeout);
+			NLA_PUT_U32(msg, IPVS_CMD_ATTR_TIMEOUT_TCP_FIN, (uint32_t)to->tcp_fin_timeout);
 		if (to->udp_timeout)
-			NLA_PUT_U32(msg, IPVS_CMD_ATTR_TIMEOUT_UDP, to->udp_timeout);
+			NLA_PUT_U32(msg, IPVS_CMD_ATTR_TIMEOUT_UDP, (uint32_t)to->udp_timeout);
 		return ipvs_nl_send_message(msg, ipvs_nl_noop_cb, NULL);
 
 nla_put_failure:
@@ -593,9 +604,9 @@ int ipvs_start_daemon(ipvs_daemon_t *dm)
 		if (!nl_daemon)
 			goto nla_put_failure;
 
-		NLA_PUT_U32(msg, IPVS_DAEMON_ATTR_STATE, dm->state);
+		NLA_PUT_S32(msg, IPVS_DAEMON_ATTR_STATE, dm->state);
 		NLA_PUT_STRING(msg, IPVS_DAEMON_ATTR_MCAST_IFN, dm->mcast_ifn);
-		NLA_PUT_U32(msg, IPVS_DAEMON_ATTR_SYNC_ID, dm->syncid);
+		NLA_PUT_S32(msg, IPVS_DAEMON_ATTR_SYNC_ID, dm->syncid);
 #ifdef _HAVE_IPVS_SYNCD_ATTRIBUTES_
 		if (dm->sync_maxlen)
 			NLA_PUT_U16(msg, IPVS_DAEMON_ATTR_SYNC_MAXLEN, dm->sync_maxlen);
@@ -619,9 +630,9 @@ nla_put_failure:
 	}
 #endif
 	memset(&dmk, 0, sizeof(dmk));
-	dmk.state = dm->state;
+	dmk.state = (int)dm->state;
 	strcpy(dmk.mcast_ifn, dm->mcast_ifn);
-	dmk.syncid = dm->syncid;
+	dmk.syncid = (int)dm->syncid;
 	return setsockopt(sockfd, IPPROTO_IP, IP_VS_SO_SET_STARTDAEMON,
 			  (char *)&dmk, sizeof(dmk));
 }
@@ -642,9 +653,9 @@ int ipvs_stop_daemon(ipvs_daemon_t *dm)
 		if (!nl_daemon)
 			goto nla_put_failure;
 
-		NLA_PUT_U32(msg, IPVS_DAEMON_ATTR_STATE, dm->state);
+		NLA_PUT_S32(msg, IPVS_DAEMON_ATTR_STATE, dm->state);
 		NLA_PUT_STRING(msg, IPVS_DAEMON_ATTR_MCAST_IFN, dm->mcast_ifn);
-		NLA_PUT_U32(msg, IPVS_DAEMON_ATTR_SYNC_ID, dm->syncid);
+		NLA_PUT_S32(msg, IPVS_DAEMON_ATTR_SYNC_ID, dm->syncid);
 
 		nla_nest_end(msg, nl_daemon);
 
@@ -656,7 +667,7 @@ nla_put_failure:
 	}
 #endif
 	memset(&dmk, 0, sizeof(dmk));
-	dmk.state = dm->state;
+	dmk.state = (int)dm->state;
 	return setsockopt(sockfd, IPPROTO_IP, IP_VS_SO_SET_STOPDAEMON,
 			  (char *)&dmk, sizeof(dmk));
 }
@@ -740,7 +751,7 @@ static int ipvs_services_parse_cb(struct nl_msg *msg, void *arg)
 	struct ip_vs_get_services_app **getp = (struct ip_vs_get_services_app **)arg;
 	struct ip_vs_get_services_app *get = (struct ip_vs_get_services_app *)*getp;
 	struct ip_vs_flags flags;
-	int i = get->user.num_services;
+	unsigned i = get->user.num_services;
 
 	if (genlmsg_parse(nlh, 0, attrs, IPVS_CMD_ATTR_MAX, ipvs_cmd_policy) != 0)
 		return -1;
@@ -779,7 +790,7 @@ static int ipvs_services_parse_cb(struct nl_msg *msg, void *arg)
 		nla_get_string(svc_attrs[IPVS_SVC_ATTR_SCHED_NAME]),
 		IP_VS_SCHEDNAME_MAXLEN);
 
-#ifdef IPVS_SVC_ATTR_PE_NAME
+#ifdef _HAVE_PE_NAME_
 	if (svc_attrs[IPVS_SVC_ATTR_PE_NAME])
 		strncpy(get->user.entrytable[i].pe_name,
 			nla_get_string(svc_attrs[IPVS_SVC_ATTR_PE_NAME]),
@@ -821,12 +832,12 @@ static int ipvs_dests_parse_cb(struct nl_msg *msg, void *arg)
 	struct nlmsghdr *nlh = nlmsg_hdr(msg);
 	struct nlattr *attrs[IPVS_CMD_ATTR_MAX + 1];
 	struct nlattr *dest_attrs[IPVS_DEST_ATTR_MAX + 1];
-#ifdef IPVS_DEST_ATTR_ADDR_FAMILY
+#if HAVE_DECL_IPVS_DEST_ATTR_ADDR_FAMILY
 	struct nlattr *attr_addr_family = NULL;
 #endif
 	struct ip_vs_get_dests_app **dp = (struct ip_vs_get_dests_app **)arg;
 	struct ip_vs_get_dests_app *d = (struct ip_vs_get_dests_app *)*dp;
-	int i = d->user.num_dests;
+	unsigned i = d->user.num_dests;
 
 	if (genlmsg_parse(nlh, 0, attrs, IPVS_CMD_ATTR_MAX, ipvs_cmd_policy) != 0)
 		return -1;
@@ -855,16 +866,16 @@ static int ipvs_dests_parse_cb(struct nl_msg *msg, void *arg)
 	       sizeof(d->user.entrytable[i].nf_addr));
 	d->user.entrytable[i].user.port = nla_get_u16(dest_attrs[IPVS_DEST_ATTR_PORT]);
 	d->user.entrytable[i].user.conn_flags = nla_get_u32(dest_attrs[IPVS_DEST_ATTR_FWD_METHOD]);
-	d->user.entrytable[i].user.weight = nla_get_u32(dest_attrs[IPVS_DEST_ATTR_WEIGHT]);
+	d->user.entrytable[i].user.weight = nla_get_s32(dest_attrs[IPVS_DEST_ATTR_WEIGHT]);
 	d->user.entrytable[i].user.u_threshold = nla_get_u32(dest_attrs[IPVS_DEST_ATTR_U_THRESH]);
 	d->user.entrytable[i].user.l_threshold = nla_get_u32(dest_attrs[IPVS_DEST_ATTR_L_THRESH]);
 	d->user.entrytable[i].user.activeconns = nla_get_u32(dest_attrs[IPVS_DEST_ATTR_ACTIVE_CONNS]);
 	d->user.entrytable[i].user.inactconns = nla_get_u32(dest_attrs[IPVS_DEST_ATTR_INACT_CONNS]);
 	d->user.entrytable[i].user.persistconns = nla_get_u32(dest_attrs[IPVS_DEST_ATTR_PERSIST_CONNS]);
-#ifdef IPVS_DEST_ATTR_ADDR_FAMILY
+#if HAVE_DECL_IPVS_DEST_ATTR_ADDR_FAMILY
 	attr_addr_family = dest_attrs[IPVS_DEST_ATTR_ADDR_FAMILY];
 	if (attr_addr_family)
-		d->entrytable[i].af = nla_get_u16(attr_addr_family);
+		d->user.entrytable[i].af = nla_get_u16(attr_addr_family);
 	else
 #endif
 		d->user.entrytable[i].af = d->af;
@@ -896,9 +907,9 @@ struct ip_vs_get_dests_app *ipvs_get_dests(ipvs_service_entry_t *svc)
 	struct ip_vs_get_dests_app *d;
 	struct ip_vs_get_dests *dk;
 	socklen_t len;
-	int i;
+	unsigned i;
 
-	len = sizeof(*d) + sizeof(ipvs_dest_entry_t) * svc->user.num_dests;
+	len = (socklen_t)(sizeof(*d) + sizeof(ipvs_dest_entry_t) * svc->user.num_dests);
 	if (!(d = MALLOC(len)))
 		return NULL;
 
@@ -956,7 +967,7 @@ ipvs_nl_dest_failure:
 		return NULL;
 	}
 
-	len = sizeof(*dk) + sizeof(struct ip_vs_dest_entry) * svc->user.num_dests;
+	len = (socklen_t)(sizeof(*dk) + sizeof(struct ip_vs_dest_entry) * svc->user.num_dests);
 	if (!(dk = MALLOC(len))) {
 		FREE(d);
 		return NULL;
@@ -1059,7 +1070,7 @@ ipvs_get_service_err2:
 	}
 	svc->af = AF_INET;
 	svc->nf_addr.ip = svc->user.addr;
-#ifdef IPVS_SVC_ATTR_PE_NAME
+#ifdef _HAVE_PE_NAME_
 	svc->pe_name[0] = '\0';
 #endif
 	return svc;
