@@ -27,6 +27,10 @@
 #include <netdb.h>
 #include <stdlib.h>
 #include <stdbool.h>
+#include <unistd.h>
+#include <sys/types.h>
+#include <pwd.h>
+#include <grp.h>
 
 #ifdef _WITH_SNMP_
 #include "snmp.h"
@@ -51,7 +55,7 @@
 /* data handlers */
 /* Global def handlers */
 static void
-use_polling_handler(vector_t *strvec)
+use_polling_handler(__attribute__((unused)) vector_t *strvec)
 {
 	global_data->linkbeat_use_polling = true;
 }
@@ -70,7 +74,7 @@ emailfrom_handler(vector_t *strvec)
 static void
 smtpto_handler(vector_t *strvec)
 {
-	global_data->smtp_connection_to = atoi(vector_slot(strvec, 1)) * TIMER_HZ;
+	global_data->smtp_connection_to = strtoul(strvec_slot(strvec, 1), NULL, 10) * TIMER_HZ;
 }
 static void
 smtpserver_handler(vector_t *strvec)
@@ -80,15 +84,15 @@ smtpserver_handler(vector_t *strvec)
 
 	/* Has a port number been specified? */
 	if (vector_size(strvec) >= 3)
-		port_str = vector_slot(strvec,2);
+		port_str = strvec_slot(strvec,2);
 
 	/* It can't be an IP address if it contains '-' or '/', and 
 	   inet_stosockaddr() modifies the string if it contains either of them */
-	if (!strpbrk(vector_slot(strvec, 1), "-/"))
-		ret = inet_stosockaddr(vector_slot(strvec, 1), port_str, &global_data->smtp_server);
+	if (!strpbrk(strvec_slot(strvec, 1), "-/"))
+		ret = inet_stosockaddr(strvec_slot(strvec, 1), port_str, &global_data->smtp_server);
 
 	if (ret < 0)
-		domain_stosockaddr(vector_slot(strvec, 1), port_str, &global_data->smtp_server);
+		domain_stosockaddr(strvec_slot(strvec, 1), port_str, &global_data->smtp_server);
 }
 static void
 smtphelo_handler(vector_t *strvec)
@@ -98,11 +102,11 @@ smtphelo_handler(vector_t *strvec)
 	if (vector_size(strvec) < 2)
 		return;
 
-	helo_name = MALLOC(strlen(vector_slot(strvec, 1)) + 1);
+	helo_name = MALLOC(strlen(strvec_slot(strvec, 1)) + 1);
 	if (!helo_name)
 		return;
 
-	strcpy(helo_name, vector_slot(strvec, 1));
+	strcpy(helo_name, strvec_slot(strvec, 1));
 	global_data->smtp_helo_name = helo_name;
 }
 static void
@@ -129,7 +133,7 @@ default_interface_handler(vector_t *strvec)
 		log_message(LOG_INFO, "default_interface requires interface name");
 		return;
 	}
-	ifp = if_get_by_ifname(vector_slot(strvec, 1));
+	ifp = if_get_by_ifname(strvec_slot(strvec, 1));
 	if (!ifp)
 		log_message(LOG_INFO, "Cannot find default interface %s", FMT_STR_VSLOT(strvec, 1));
 	else
@@ -140,8 +144,8 @@ default_interface_handler(vector_t *strvec)
 static void
 lvs_timeouts(vector_t *strvec)
 {
-	int val;
-	int i;
+	long val;
+	size_t i;
 	char *endptr;
 
 	if (vector_size(strvec) < 3) {
@@ -150,42 +154,42 @@ lvs_timeouts(vector_t *strvec)
 	}
 
 	for (i = 1; i < vector_size(strvec); i++) {
-		if (!strcmp(vector_slot(strvec, i), "tcp")) {
+		if (!strcmp(strvec_slot(strvec, i), "tcp")) {
 			if (i == vector_size(strvec) - 1) {
 				log_message(LOG_INFO, "No value specified for lvs_timout tcp - ignoring");
 				continue;
 			}
-			val = strtol(vector_slot(strvec, i+1), &endptr, 10);
+			val = strtol(strvec_slot(strvec, i+1), &endptr, 10);
 			if (*endptr != '\0' || val < 0 || val > LVS_MAX_TIMEOUT)
 				log_message(LOG_INFO, "Invalid lvs_timeout tcp (%s) - ignoring", FMT_STR_VSLOT(strvec, i+1));
 			else
-				global_data->lvs_tcp_timeout = val;
+				global_data->lvs_tcp_timeout = (int)val;
 			i++;	/* skip over value */
 			continue;
 		}
-		if (!strcmp(vector_slot(strvec, i), "tcpfin")) {
+		if (!strcmp(strvec_slot(strvec, i), "tcpfin")) {
 			if (i == vector_size(strvec) - 1) {
 				log_message(LOG_INFO, "No value specified for lvs_timeout tcpfin - ignoring");
 				continue;
 			}
-			val = strtol(vector_slot(strvec, i+1), &endptr, 10);
+			val = strtol(strvec_slot(strvec, i+1), &endptr, 10);
 			if (*endptr != '\0' || val < 1 || val > LVS_MAX_TIMEOUT)
 				log_message(LOG_INFO, "Invalid lvs_timeout tcpfin (%s) - ignoring", FMT_STR_VSLOT(strvec, i+1));
 			else
-				global_data->lvs_tcpfin_timeout = val;
+				global_data->lvs_tcpfin_timeout = (int)val;
 			i++;	/* skip over value */
 			continue;
 		}
-		if (!strcmp(vector_slot(strvec, i), "udp")) {
+		if (!strcmp(strvec_slot(strvec, i), "udp")) {
 			if (i == vector_size(strvec) - 1) {
 				log_message(LOG_INFO, "No value specified for lvs_timeout udp - ignoring");
 				continue;
 			}
-			val = strtol(vector_slot(strvec, i+1), &endptr, 10);
+			val = strtol(strvec_slot(strvec, i+1), &endptr, 10);
 			if (*endptr != '\0' || val < 1 || val > LVS_MAX_TIMEOUT)
 				log_message(LOG_INFO, "Invalid lvs_timeout udp (%s) - ignoring", FMT_STR_VSLOT(strvec, i+1));
 			else
-				global_data->lvs_udp_timeout = val;
+				global_data->lvs_udp_timeout = (int)val;
 			i++;	/* skip over value */
 			continue;
 		}
@@ -196,8 +200,8 @@ lvs_timeouts(vector_t *strvec)
 static void
 lvs_syncd_handler(vector_t *strvec)
 {
-	int val;
-	int i;
+	unsigned long val;
+	size_t i;
 	char *endptr;
 
 	if (global_data->lvs_syncd.ifname) {
@@ -212,85 +216,85 @@ lvs_syncd_handler(vector_t *strvec)
 
 	global_data->lvs_syncd.ifname = set_value(strvec);
 
-	global_data->lvs_syncd.vrrp_name = MALLOC(strlen(vector_slot(strvec, 2)) + 1);
+	global_data->lvs_syncd.vrrp_name = MALLOC(strlen(strvec_slot(strvec, 2)) + 1);
 	if (!global_data->lvs_syncd.vrrp_name)
 		return;
-	strcpy(global_data->lvs_syncd.vrrp_name, vector_slot(strvec, 2));
+	strcpy(global_data->lvs_syncd.vrrp_name, strvec_slot(strvec, 2));
 
 	/* This is maintained for backwards compatibility, prior to adding "id" option */
 	if (vector_size(strvec) >= 4 && isdigit(FMT_STR_VSLOT(strvec, 3)[0])) {
 		log_message(LOG_INFO, "Please use keyword \"id\" before lvs_sync_daemon syncid value");
-		val = strtol(vector_slot(strvec,3), &endptr, 10);
-		if (*endptr || val < 0 || val > 255)
+		val = strtoul(strvec_slot(strvec,3), &endptr, 10);
+		if (*endptr || val > 255)
 			log_message(LOG_INFO, "Invalid syncid (%s) - defaulting to vrid", FMT_STR_VSLOT(strvec, 3));
 		else
-			global_data->lvs_syncd.syncid = val;
+			global_data->lvs_syncd.syncid = (unsigned)val;
 		i = 4;
 	}
 	else
 		i = 3;
 
 	for ( ; i < vector_size(strvec); i++) {
-		if (!strcmp(vector_slot(strvec, i), "id")) {
+		if (!strcmp(strvec_slot(strvec, i), "id")) {
 			if (i == vector_size(strvec) - 1) {
 				log_message(LOG_INFO, "No value specified for lvs_sync_daemon id, defaulting to vrid");
 				continue;
 			}
-			val = strtol(vector_slot(strvec, i+1), &endptr, 10);
-			if (*endptr != '\0' || val < 0 || val > 255)
+			val = strtoul(strvec_slot(strvec, i+1), &endptr, 10);
+			if (*endptr != '\0' || val > 255)
 				log_message(LOG_INFO, "Invalid syncid (%s) - defaulting to vrid", FMT_STR_VSLOT(strvec, i+1));
 			else
-				global_data->lvs_syncd.syncid = val;
+				global_data->lvs_syncd.syncid = (unsigned)val;
 			i++;	/* skip over value */
 			continue;
 		}
 #ifdef _HAVE_IPVS_SYNCD_ATTRIBUTES_
-		if (!strcmp(vector_slot(strvec, i), "maxlen")) {
+		if (!strcmp(strvec_slot(strvec, i), "maxlen")) {
 			if (i == vector_size(strvec) - 1) {
 				log_message(LOG_INFO, "No value specified for lvs_sync_daemon maxlen - ignoring");
 				continue;
 			}
-			val = strtol(vector_slot(strvec, i+1), &endptr, 10);
-			if (*endptr != '\0' || val < 1 || val > 65535 - 20 - 8)
+			val = strtoul(strvec_slot(strvec, i+1), &endptr, 10);
+			if (*endptr != '\0' || !val || val > 65535 - 20 - 8)
 				log_message(LOG_INFO, "Invalid lvs_sync_daemon maxlen (%s) - ignoring", FMT_STR_VSLOT(strvec, i+1));
 			else
-				global_data->lvs_syncd.sync_maxlen = val;
+				global_data->lvs_syncd.sync_maxlen = (uint16_t)val;
 			i++;	/* skip over value */
 			continue;
 		}
-		if (!strcmp(vector_slot(strvec, i), "port")) {
+		if (!strcmp(strvec_slot(strvec, i), "port")) {
 			if (i == vector_size(strvec) - 1) {
 				log_message(LOG_INFO, "No value specified for lvs_sync_daemon port - ignoring");
 				continue;
 			}
-			val = strtol(vector_slot(strvec, i+1), &endptr, 10);
-			if (*endptr != '\0' || val < 1 || val > 65535)
+			val = strtoul(strvec_slot(strvec, i+1), &endptr, 10);
+			if (*endptr != '\0' || !val || val > 65535)
 				log_message(LOG_INFO, "Invalid lvs_sync_daemon port (%s) - ignoring", FMT_STR_VSLOT(strvec, i+1));
 			else
-				global_data->lvs_syncd.mcast_port = val;
+				global_data->lvs_syncd.mcast_port = (uint16_t)val;
 			i++;	/* skip over value */
 			continue;
 		}
-		if (!strcmp(vector_slot(strvec, i), "ttl")) {
+		if (!strcmp(strvec_slot(strvec, i), "ttl")) {
 			if (i == vector_size(strvec) - 1) {
 				log_message(LOG_INFO, "No value specified for lvs_sync_daemon ttl - ignoring");
 				continue;
 			}
-			val = strtol(vector_slot(strvec, i+1), &endptr, 10);
-			if (*endptr != '\0' || val < 1 || val > 255)
+			val = strtoul(strvec_slot(strvec, i+1), &endptr, 10);
+			if (*endptr != '\0' || !val || val > 255)
 				log_message(LOG_INFO, "Invalid lvs_sync_daemon ttl (%s) - ignoring", FMT_STR_VSLOT(strvec, i+1));
 			else
-				global_data->lvs_syncd.mcast_ttl = val;
+				global_data->lvs_syncd.mcast_ttl = (uint8_t)val;
 			i++;	/* skip over value */
 			continue;
 		}
-		if (!strcmp(vector_slot(strvec, i), "group")) {
+		if (!strcmp(strvec_slot(strvec, i), "group")) {
 			if (i == vector_size(strvec) - 1) {
 				log_message(LOG_INFO, "No value specified for lvs_sync_daemon group - ignoring");
 				continue;
 			}
 
-			if (inet_stosockaddr(vector_slot(strvec, i+1), NULL, &global_data->lvs_syncd.mcast_group) < 0)
+			if (inet_stosockaddr(strvec_slot(strvec, i+1), NULL, &global_data->lvs_syncd.mcast_group) < 0)
 				log_message(LOG_INFO, "Invalid lvs_sync_daemon group (%s) - ignoring", FMT_STR_VSLOT(strvec, i+1));
 
 			if ((global_data->lvs_syncd.mcast_group.ss_family == AF_INET  && !IN_MULTICAST(htonl(((struct sockaddr_in *)&global_data->lvs_syncd.mcast_group)->sin_addr.s_addr))) ||
@@ -308,7 +312,7 @@ lvs_syncd_handler(vector_t *strvec)
 }
 #endif
 static void
-lvs_flush_handler(vector_t *strvec)
+lvs_flush_handler(__attribute__((unused)) vector_t *strvec)
 {
 	global_data->lvs_flush = true;
 }
@@ -320,7 +324,7 @@ vrrp_mcast_group4_handler(vector_t *strvec)
 	struct sockaddr_storage *mcast = &global_data->vrrp_mcast_group4;
 	int ret;
 
-	ret = inet_stosockaddr(vector_slot(strvec, 1), 0, mcast);
+	ret = inet_stosockaddr(strvec_slot(strvec, 1), 0, mcast);
 	if (ret < 0) {
 		log_message(LOG_ERR, "Configuration error: Cant parse vrrp_mcast_group4 [%s]. Skipping"
 				   , FMT_STR_VSLOT(strvec, 1));
@@ -332,7 +336,7 @@ vrrp_mcast_group6_handler(vector_t *strvec)
 	struct sockaddr_storage *mcast = &global_data->vrrp_mcast_group6;
 	int ret;
 
-	ret = inet_stosockaddr(vector_slot(strvec, 1), 0, mcast);
+	ret = inet_stosockaddr(strvec_slot(strvec, 1), 0, mcast);
 	if (ret < 0) {
 		log_message(LOG_ERR, "Configuration error: Cant parse vrrp_mcast_group6 [%s]. Skipping"
 				   , FMT_STR_VSLOT(strvec, 1));
@@ -341,52 +345,49 @@ vrrp_mcast_group6_handler(vector_t *strvec)
 static void
 vrrp_garp_delay_handler(vector_t *strvec)
 {
-	global_data->vrrp_garp_delay = atoi(vector_slot(strvec, 1)) * TIMER_HZ;
+	global_data->vrrp_garp_delay = (unsigned)strtoul(strvec_slot(strvec, 1), NULL, 10) * TIMER_HZ;
 }
 static void
 vrrp_garp_rep_handler(vector_t *strvec)
 {
-	global_data->vrrp_garp_rep = atoi(vector_slot(strvec, 1));
+	global_data->vrrp_garp_rep = (unsigned)strtoul(strvec_slot(strvec, 1), NULL, 10);
 	if (global_data->vrrp_garp_rep < 1)
 		global_data->vrrp_garp_rep = 1;
 }
 static void
 vrrp_garp_refresh_handler(vector_t *strvec)
 {
-	global_data->vrrp_garp_refresh.tv_sec = atoi(vector_slot(strvec, 1));
+	global_data->vrrp_garp_refresh.tv_sec = (unsigned)strtoul(strvec_slot(strvec, 1), NULL, 10);
 }
 static void
 vrrp_garp_refresh_rep_handler(vector_t *strvec)
 {
-	global_data->vrrp_garp_refresh_rep = atoi(vector_slot(strvec, 1));
+	global_data->vrrp_garp_refresh_rep = (unsigned)strtoul(strvec_slot(strvec, 1), NULL, 10);
 	if (global_data->vrrp_garp_refresh_rep < 1)
 		global_data->vrrp_garp_refresh_rep = 1;
 }
 static void
 vrrp_garp_lower_prio_delay_handler(vector_t *strvec)
 {
-	global_data->vrrp_garp_lower_prio_delay = atoi(vector_slot(strvec, 1)) * TIMER_HZ;
+	global_data->vrrp_garp_lower_prio_delay = (unsigned)strtoul(strvec_slot(strvec, 1), NULL, 10) * TIMER_HZ;
 }
 static void
 vrrp_garp_lower_prio_rep_handler(vector_t *strvec)
 {
-	global_data->vrrp_garp_lower_prio_rep = atoi(vector_slot(strvec, 1));
-	/* Allow 0 GARP messages to be sent */
-	if (global_data->vrrp_garp_lower_prio_rep < 0)
-		global_data->vrrp_garp_lower_prio_rep = 0;
+	global_data->vrrp_garp_lower_prio_rep = (unsigned)strtoul(strvec_slot(strvec, 1), NULL, 10);
 }
 static void
 vrrp_garp_interval_handler(vector_t *strvec)
 {
-	global_data->vrrp_garp_interval = atof(vector_slot(strvec, 1)) * 1000000;
-	if (global_data->vrrp_garp_interval >= 1000000)
+	global_data->vrrp_garp_interval = (unsigned)(atof(strvec_slot(strvec, 1)) * TIMER_HZ);
+	if (global_data->vrrp_garp_interval >= 1 * TIMER_HZ)
 		log_message(LOG_INFO, "The vrrp_garp_interval is very large - %s seconds", FMT_STR_VSLOT(strvec, 1));
 }
 static void
 vrrp_gna_interval_handler(vector_t *strvec)
 {
-	global_data->vrrp_gna_interval = atof(vector_slot(strvec, 1)) * 1000000;
-	if (global_data->vrrp_gna_interval >= 1000000)
+	global_data->vrrp_gna_interval = (unsigned)(atof(strvec_slot(strvec, 1)) * TIMER_HZ);
+	if (global_data->vrrp_gna_interval >= 1 * TIMER_HZ)
 		log_message(LOG_INFO, "The vrrp_gna_interval is very large - %s seconds", FMT_STR_VSLOT(strvec, 1));
 }
 static void
@@ -395,7 +396,7 @@ vrrp_lower_prio_no_advert_handler(vector_t *strvec)
 	int res;
 
 	if (vector_size(strvec) >= 2) {
-		res = check_true_false(vector_slot(strvec,1));
+		res = check_true_false(strvec_slot(strvec,1));
 		if (res < 0)
 			log_message(LOG_INFO, "Invalid value for vrrp_lower_prio_no_advert specified");
 		else
@@ -410,18 +411,18 @@ vrrp_iptables_handler(vector_t *strvec)
 	global_data->vrrp_iptables_inchain[0] = '\0';
 	global_data->vrrp_iptables_outchain[0] = '\0';
 	if (vector_size(strvec) >= 2) {
-		if (strlen(vector_slot(strvec,1)) >= sizeof(global_data->vrrp_iptables_inchain)-1) {
+		if (strlen(strvec_slot(strvec,1)) >= sizeof(global_data->vrrp_iptables_inchain)-1) {
 			log_message(LOG_INFO, "VRRP Error : iptables in chain name too long - ignored");
 			return;
 		}
-		strcpy(global_data->vrrp_iptables_inchain, vector_slot(strvec,1));
+		strcpy(global_data->vrrp_iptables_inchain, strvec_slot(strvec,1));
 	}
 	if (vector_size(strvec) >= 3) {
-		if (strlen(vector_slot(strvec,2)) >= sizeof(global_data->vrrp_iptables_outchain)-1) {
+		if (strlen(strvec_slot(strvec,2)) >= sizeof(global_data->vrrp_iptables_outchain)-1) {
 			log_message(LOG_INFO, "VRRP Error : iptables out chain name too long - ignored");
 			return;
 		}
-		strcpy(global_data->vrrp_iptables_outchain, vector_slot(strvec,2));
+		strcpy(global_data->vrrp_iptables_outchain, strvec_slot(strvec,2));
 	}
 }
 #ifdef _HAVE_LIBIPSET_
@@ -431,11 +432,11 @@ vrrp_ipsets_handler(vector_t *strvec)
 	size_t len;
 
 	if (vector_size(strvec) >= 2) {
-		if (strlen(vector_slot(strvec,1)) >= sizeof(global_data->vrrp_ipset_address)-1) {
+		if (strlen(strvec_slot(strvec,1)) >= sizeof(global_data->vrrp_ipset_address)-1) {
 			log_message(LOG_INFO, "VRRP Error : ipset address name too long - ignored");
 			return;
 		}
-		strcpy(global_data->vrrp_ipset_address, vector_slot(strvec,1));
+		strcpy(global_data->vrrp_ipset_address, strvec_slot(strvec,1));
 	}
 	else {
 		global_data->using_ipsets = false;
@@ -443,11 +444,11 @@ vrrp_ipsets_handler(vector_t *strvec)
 	}
 
 	if (vector_size(strvec) >= 3) {
-		if (strlen(vector_slot(strvec,2)) >= sizeof(global_data->vrrp_ipset_address6)-1) {
+		if (strlen(strvec_slot(strvec,2)) >= sizeof(global_data->vrrp_ipset_address6)-1) {
 			log_message(LOG_INFO, "VRRP Error : ipset IPv6 address name too long - ignored");
 			return;
 		}
-		strcpy(global_data->vrrp_ipset_address6, vector_slot(strvec,2));
+		strcpy(global_data->vrrp_ipset_address6, strvec_slot(strvec,2));
 	}
 	else {
 		/* No second set specified, copy first name and add "6" */
@@ -456,11 +457,11 @@ vrrp_ipsets_handler(vector_t *strvec)
 		strcat(global_data->vrrp_ipset_address6, "6");
 	}
 	if (vector_size(strvec) >= 4) {
-		if (strlen(vector_slot(strvec,3)) >= sizeof(global_data->vrrp_ipset_address_iface6)-1) {
+		if (strlen(strvec_slot(strvec,3)) >= sizeof(global_data->vrrp_ipset_address_iface6)-1) {
 			log_message(LOG_INFO, "VRRP Error : ipset IPv6 address_iface name too long - ignored");
 			return;
 		}
-		strcpy(global_data->vrrp_ipset_address_iface6, vector_slot(strvec,3));
+		strcpy(global_data->vrrp_ipset_address_iface6, strvec_slot(strvec,3));
 	}
 	else {
 		/* No third set specified, copy second name and add "_if6" */
@@ -476,7 +477,7 @@ vrrp_ipsets_handler(vector_t *strvec)
 static void
 vrrp_version_handler(vector_t *strvec)
 {
-	uint8_t version = atoi(vector_slot(strvec, 1));
+	uint8_t version = (uint8_t)strtoul(strvec_slot(strvec, 1), NULL, 10);
 	if (VRRP_IS_BAD_VERSION(version)) {
 		log_message(LOG_INFO, "VRRP Error : Version not valid !");
 		log_message(LOG_INFO, "             must be between either 2 or 3. reconfigure !");
@@ -485,17 +486,17 @@ vrrp_version_handler(vector_t *strvec)
 	global_data->vrrp_version = version;
 }
 static void
-vrrp_check_unicast_src_handler(vector_t *strvec)
+vrrp_check_unicast_src_handler(__attribute__((unused)) vector_t *strvec)
 {
 	global_data->vrrp_check_unicast_src = 1;
 }
 static void
-vrrp_check_adv_addr_handler(vector_t *strvec)
+vrrp_check_adv_addr_handler(__attribute__((unused)) vector_t *strvec)
 {
 	global_data->vrrp_skip_check_adv_addr = 1;
 }
 static void
-vrrp_strict_handler(vector_t *strvec)
+vrrp_strict_handler(__attribute__((unused)) vector_t *strvec)
 {
 	global_data->vrrp_strict = 1;
 }
@@ -509,16 +510,16 @@ vrrp_prio_handler(vector_t *strvec)
 		return;
 	}
 
-	priority = atoi(vector_slot(strvec, 1));
+	priority = atoi(strvec_slot(strvec, 1));
 	if (priority < -20 || priority > 19) {
 		log_message(LOG_INFO, "Invalid vrrp process priority specified");
 		return;
 	}
 
-	global_data->vrrp_process_priority = priority;
+	global_data->vrrp_process_priority = (int8_t)priority;
 }
 static void
-vrrp_no_swap_handler(vector_t *strvec)
+vrrp_no_swap_handler(__attribute__((unused)) vector_t *strvec)
 {
 	global_data->vrrp_no_swap = true;
 }
@@ -534,16 +535,16 @@ checker_prio_handler(vector_t *strvec)
 		return;
 	}
 
-	priority = atoi(vector_slot(strvec, 1));
+	priority = atoi(strvec_slot(strvec, 1));
 	if (priority < -20 || priority > 19) {
 		log_message(LOG_INFO, "Invalid checker process priority specified");
 		return;
 	}
 
-	global_data->checker_process_priority = priority;
+	global_data->checker_process_priority = (int8_t)priority;
 }
 static void
-checker_no_swap_handler(vector_t *strvec)
+checker_no_swap_handler(__attribute__((unused)) vector_t *strvec)
 {
 	global_data->checker_no_swap = true;
 }
@@ -562,7 +563,7 @@ snmp_socket_handler(vector_t *strvec)
 		return;
 	}
 
-	if (strlen(vector_slot(strvec,1)) > PATH_MAX - 1) {
+	if (strlen(strvec_slot(strvec,1)) > PATH_MAX - 1) {
 		log_message(LOG_INFO, "SNMP error : snmp socket name too long - ignored");
 		return;
 	}
@@ -572,24 +573,24 @@ snmp_socket_handler(vector_t *strvec)
 		return;
 	}
 
-	global_data->snmp_socket = MALLOC(strlen(vector_slot(strvec, 1) + 1));
-	strcpy(global_data->snmp_socket, vector_slot(strvec,1));
+	global_data->snmp_socket = MALLOC(strlen(strvec_slot(strvec, 1) + 1));
+	strcpy(global_data->snmp_socket, strvec_slot(strvec,1));
 }
 static void
-trap_handler(vector_t *strvec)
+trap_handler(__attribute__((unused)) vector_t *strvec)
 {
 	global_data->enable_traps = true;
 }
 #ifdef _WITH_SNMP_KEEPALIVED_
 static void
-snmp_keepalived_handler(vector_t *strvec)
+snmp_keepalived_handler(__attribute__((unused)) vector_t *strvec)
 {
 	global_data->enable_snmp_keepalived = true;
 }
 #endif
 #ifdef _WITH_SNMP_RFC_
 static void
-snmp_rfc_handler(vector_t *strvec)
+snmp_rfc_handler(__attribute__((unused)) vector_t *strvec)
 {
 	global_data->enable_snmp_rfcv2 = true;
 	global_data->enable_snmp_rfcv3 = true;
@@ -597,21 +598,21 @@ snmp_rfc_handler(vector_t *strvec)
 #endif
 #ifdef _WITH_SNMP_RFCV2_
 static void
-snmp_rfcv2_handler(vector_t *strvec)
+snmp_rfcv2_handler(__attribute__((unused)) vector_t *strvec)
 {
 	global_data->enable_snmp_rfcv2 = true;
 }
 #endif
 #ifdef _WITH_SNMP_RFCV3_
 static void
-snmp_rfcv3_handler(vector_t *strvec)
+snmp_rfcv3_handler(__attribute__((unused)) vector_t *strvec)
 {
 	global_data->enable_snmp_rfcv3 = true;
 }
 #endif
 #ifdef _WITH_SNMP_CHECKER_
 static void
-snmp_checker_handler(vector_t *strvec)
+snmp_checker_handler(__attribute__((unused)) vector_t *strvec)
 {
 	global_data->enable_snmp_checker = true;
 }
@@ -650,6 +651,20 @@ net_namespace_handler(vector_t *strvec)
 	}
 #endif
 }
+
+static void
+namespace_ipsets_handler(__attribute__((unused)) vector_t *strvec)
+{
+	namespace_with_ipsets = true;
+}
+#endif
+
+#ifdef _WITH_DBUS_
+static void
+enable_dbus_handler(__attribute__((unused)) vector_t *strvec)
+{
+	global_data->enable_dbus = true;
+}
 #endif
 
 static void
@@ -666,11 +681,76 @@ instance_handler(vector_t *strvec)
 }
 
 static void
-use_pid_dir_handler(vector_t *strvec)
+use_pid_dir_handler(__attribute__((unused)) vector_t *strvec)
 {
 	use_pid_dir = true;
 }
 
+bool
+set_script_uid_gid(vector_t *strvec, unsigned keyword_offset, uid_t *uid_p, gid_t *gid_p)
+{
+	char *username;
+	char *groupname;
+	uid_t uid;
+	gid_t gid;
+	struct passwd pwd;
+	struct passwd *pwd_p;
+	struct group grp;
+	struct group *grp_p;
+	int ret;
+	char buf[getpwnam_buf_len];
+
+	username = strvec_slot(strvec, keyword_offset);
+
+	if ((ret = getpwnam_r(username, &pwd, buf, sizeof(buf), &pwd_p))) {
+		log_message(LOG_INFO, "Unable to resolve script username '%s' - ignoring", username);
+		return true;
+	}
+	if (!pwd_p) {
+		log_message(LOG_INFO, "Script user '%s' does not exist", username);
+		return true;
+	}
+
+	uid = pwd.pw_uid;
+	gid = pwd.pw_gid;
+
+	if (vector_size(strvec) > keyword_offset + 1) {
+		groupname = strvec_slot(strvec, keyword_offset + 1);
+		if ((ret = getgrnam_r(groupname, &grp, buf, sizeof(buf), &grp_p))) {
+			log_message(LOG_INFO, "Unable to resolve script group name '%s' - ignoring", groupname);
+			return true;
+		}
+		if (!grp_p) {
+			log_message(LOG_INFO, "Script group '%s' does not exist", groupname);
+			return true;
+		}
+		gid = grp.gr_gid;
+	}
+
+	*uid_p = uid;
+	*gid_p = gid;
+
+	return false;
+}
+
+static void
+script_user_handler(vector_t *strvec)
+{
+	if (vector_size(strvec) < 2) {
+		log_message(LOG_INFO, "No script username specified");
+		return;
+	}
+
+	if (set_script_uid_gid(strvec, 1, &default_script_uid, &default_script_gid))
+		log_message(LOG_INFO, "Error setting global script uid/gid");
+}
+
+static void
+script_security_handler(__attribute__((unused)) vector_t *strvec)
+{
+	global_data->script_security = true;
+}
+
 void
 init_global_keywords(bool global_active)
 {
@@ -678,6 +758,7 @@ init_global_keywords(bool global_active)
 	install_keyword_root("linkbeat_use_polling", use_polling_handler, global_active);
 #if HAVE_DECL_CLONE_NEWNET
 	install_keyword_root("net_namespace", &net_namespace_handler, !global_active);
+	install_keyword_root("namespace_with_ipsets", &namespace_ipsets_handler, !global_active);
 #endif
 	install_keyword_root("use_pid_dir", &use_pid_dir_handler, !global_active);
 	install_keyword_root("instance", &instance_handler, !global_active);
@@ -742,4 +823,9 @@ init_global_keywords(bool global_active)
 	install_keyword("enable_snmp_checker", &snmp_checker_handler);
 #endif
 #endif
+#ifdef _WITH_DBUS_
+	install_keyword("enable_dbus", &enable_dbus_handler);
+#endif
+	install_keyword("script_user", &script_user_handler);
+	install_keyword("enable_script_security", &script_security_handler);
 }
