@@ -31,7 +31,7 @@
 #include <net-snmp/agent/agent_sysORTable.h>
 
 static int
-snmp_keepalived_log(int major, int minor, void *serverarg, void *clientarg)
+snmp_keepalived_log(__attribute__((unused)) int major, __attribute__((unused)) int minor, void *serverarg, __attribute__((unused)) void *clientarg)
 {
 	struct snmp_log_message *slm = (struct snmp_log_message*)serverarg;
 	log_message(slm->priority, "%s", slm->msg);
@@ -59,7 +59,7 @@ snmp_header_list_table(struct variable *vp, oid *name, size_t *length,
 {
 	element e;
 	void *scr;
-	unsigned int target, current;
+	oid target, current;
 
 	if (header_simple_table(vp, name, length, exact, var_len, write_method, -1))
 		return NULL;
@@ -86,7 +86,7 @@ snmp_header_list_table(struct variable *vp, oid *name, size_t *length,
 		name[*length - 1] = current;
 		return scr;
 	}
-	/* No macth found at end */
+	/* No match found at end */
 	return NULL;
 }
 
@@ -103,6 +103,7 @@ enum snmp_global_magic {
 	SNMP_LVSFLUSH,
 	SNMP_IPVS_64BIT_STATS,
 	SNMP_NET_NAMESPACE,
+	SNMP_DBUS,
 };
 
 static u_char*
@@ -116,7 +117,7 @@ snmp_scalar(struct variable *vp, oid *name, size_t *length,
 
 	switch (vp->magic) {
 	case SNMP_KEEPALIVEDVERSION:
-		*var_len = sizeof(version_string) - 1;
+		*var_len = strlen(version_string);
 		return (u_char *)version_string;
 	case SNMP_ROUTERID:
 		if (!global_data->router_id) return NULL;
@@ -149,9 +150,11 @@ snmp_scalar(struct variable *vp, oid *name, size_t *length,
 	case SNMP_LINKBEAT:
 		long_ret = global_data->linkbeat_use_polling?2:1;
 		return (u_char *)&long_ret;
+#ifdef _WITH_LVS_
 	case SNMP_LVSFLUSH:
 		long_ret = global_data->lvs_flush?1:2;
 		return (u_char *)&long_ret;
+#endif
 	case SNMP_IPVS_64BIT_STATS:
 #ifdef _WITH_LVS_64BIT_STATS_
 		long_ret = 1;
@@ -168,6 +171,14 @@ snmp_scalar(struct variable *vp, oid *name, size_t *length,
 #endif
 		*var_len = 0;
 		return (u_char *)"";
+	case SNMP_DBUS:
+#ifdef _WITH_DBUS_
+		if (global_data->enable_dbus)
+			long_ret = 1;
+		else
+#endif
+			long_ret = 2;
+		return (u_char *)&long_ret;
 	default:
 		break;
 	}
@@ -219,11 +230,14 @@ static struct variable8 global_vars[] = {
 	{SNMP_IPVS_64BIT_STATS, ASN_INTEGER, RONLY, snmp_scalar, 1, {7}},
 #endif
 	{SNMP_NET_NAMESPACE, ASN_OCTET_STR, RONLY, snmp_scalar, 1, {8}},
+#ifdef _WITH_DBUS_
+	{SNMP_DBUS, ASN_INTEGER, RONLY, snmp_scalar, 1, {9}},
+#endif
 };
 
 static int
-snmp_setup_session_cb(int majorID, int minorID,
-		      void *serverarg, void *clientarg)
+snmp_setup_session_cb(__attribute__((unused)) int majorID, __attribute__((unused)) int minorID,
+		      void *serverarg, __attribute__((unused)) void *clientarg)
 {
 	netsnmp_session *sess = serverarg;
 	if (serverarg == NULL)
@@ -238,8 +252,8 @@ snmp_setup_session_cb(int majorID, int minorID,
 	return 0;
 }
 
-void snmp_register_mib(oid *myoid, int len, const char *name,
-		       struct variable *variables, int varsize, int varlen)
+void snmp_register_mib(oid *myoid, size_t len, const char *name,
+		       struct variable *variables, size_t varsize, size_t varlen)
 {
 	char name_buf[80];
 
@@ -252,7 +266,7 @@ void snmp_register_mib(oid *myoid, int len, const char *name,
 }
 
 void
-snmp_unregister_mib(oid *myoid, int len)
+snmp_unregister_mib(oid *myoid, size_t len)
 {
 	unregister_sysORTable(myoid, len);
 }
