@@ -24,8 +24,12 @@
 #ifndef _VRRP_IPTABLES_CALLS_H
 #define	_VRRP_IPTABLES_CALLS_H
 
+#include <limits.h>
+
 #include "vrrp_ipaddress.h"
 
+#define	APPEND_RULE	UINT_MAX
+
 #ifdef _HAVE_LIBIPSET_
 int load_mod_xt_set(void);
 #endif
@@ -33,12 +37,12 @@ int load_mod_xt_set(void);
 struct iptc_handle* ip4tables_open ( const char*);
 int ip4tables_close ( struct iptc_handle*, int);
 int ip4tables_is_chain(struct iptc_handle*, const char*);
-int ip4tables_process_entry( struct iptc_handle* handle, const char* chain_name, int rulenum, const char* target_name, const ip_address_t* src_ip_address, const ip_address_t* dst_ip_address, const char* in_iface, const char* out_iface, uint16_t protocol, uint16_t type, int cmd, bool force);
+int ip4tables_process_entry( struct iptc_handle* handle, const char* chain_name, unsigned int rulenum, const char* target_name, const ip_address_t* src_ip_address, const ip_address_t* dst_ip_address, const char* in_iface, const char* out_iface, uint16_t protocol, uint8_t type, int cmd, bool force);
 struct ip6tc_handle* ip6tables_open ( const char* tablename );
 int ip6tables_close ( struct ip6tc_handle* handle, int updated );
 int ip6tables_is_chain(struct ip6tc_handle* handle, const char* chain_name);
-int ip6tables_process_entry( struct ip6tc_handle* handle, const char* chain_name, int rulenum, const char* target_name, const ip_address_t* src_ip_address, const ip_address_t* dst_ip_address, const char* in_iface, const char* out_iface, uint16_t protocol, uint16_t type, int cmd, bool force);
-int ip4tables_add_rules(struct iptc_handle* handle, const char* chain_name, int rulenum, int dim, int src_dst, const char* target_name, const char* set_name, uint16_t protocol, int param, int cmd, bool ignore_errors);
-int ip6tables_add_rules(struct ip6tc_handle* handle, const char* chain_name, int rulenum, int dim, int src_dst, const char* target_name, const char* set_name, uint16_t protocol, int param, int cmd, bool ignore_errors);
+int ip6tables_process_entry( struct ip6tc_handle* handle, const char* chain_name, unsigned int rulenum, const char* target_name, const ip_address_t* src_ip_address, const ip_address_t* dst_ip_address, const char* in_iface, const char* out_iface, uint16_t protocol, uint8_t type, int cmd, bool force);
+int ip4tables_add_rules(struct iptc_handle* handle, const char* chain_name, unsigned int rulenum, uint8_t dim, uint8_t src_dst, const char* target_name, const char* set_name, uint16_t protocol, uint8_t param, int cmd, bool ignore_errors);
+int ip6tables_add_rules(struct ip6tc_handle* handle, const char* chain_name, unsigned int rulenum, uint8_t dim, uint8_t src_dst, const char* target_name, const char* set_name, uint16_t protocol, uint8_t param, int cmd, bool ignore_errors);
 
 #endif
