@@ -33,6 +33,7 @@
 #include "vrrp_vmac.h"
 #endif
 #include "vrrp.h"
+#include "main.h"
 #include "global_data.h"
 #include "global_parser.h"
 #ifdef _WITH_LVS_
@@ -42,27 +43,28 @@
 #include "parser.h"
 #include "memory.h"
 #include "bitops.h"
+#include "notify.h"
 
 /* Static addresses handler */
 static void
-static_addresses_handler(vector_t *strvec)
+static_addresses_handler(__attribute__((unused)) vector_t *strvec)
 {
-	alloc_value_block(strvec, alloc_saddress);
+	alloc_value_block(alloc_saddress);
 }
 
 #ifdef _HAVE_FIB_ROUTING_
 /* Static routes handler */
 static void
-static_routes_handler(vector_t *strvec)
+static_routes_handler(__attribute__((unused)) vector_t *strvec)
 {
-	alloc_value_block(strvec, alloc_sroute);
+	alloc_value_block(alloc_sroute);
 }
 
 /* Static rules handler */
 static void
-static_rules_handler(vector_t *strvec)
+static_rules_handler(__attribute__((unused)) vector_t *strvec)
 {
-	alloc_value_block(strvec, alloc_srule);
+	alloc_value_block(alloc_srule);
 }
 #endif
 
@@ -81,7 +83,7 @@ vrrp_sync_group_handler(vector_t *strvec)
 		return;
 	}
 
-	gname = vector_slot(strvec, 1);
+	gname = strvec_slot(strvec, 1);
 
 	/* check group doesn't already exist */
 	if (!LIST_ISEMPTY(vrrp_data->vrrp_sync_group)) {
@@ -111,42 +113,56 @@ vrrp_group_handler(vector_t *strvec)
 
 	vgroup->iname = read_value_block(strvec);
 }
+
+static inline notify_script_t*
+set_vrrp_notify_script(vector_t *strvec)
+{
+	notify_script_t *script = notify_script_init(strvec, default_script_uid, default_script_gid);
+
+	if (vector_size(strvec) > 2) {
+		if (set_script_uid_gid(strvec, 2, &script->uid, &script->gid))
+			log_message(LOG_INFO, "Invalid user/group for notify script %s", script->name);
+	}
+
+	return script;
+}
+
 static void
 vrrp_gnotify_backup_handler(vector_t *strvec)
 {
 	vrrp_sgroup_t *vgroup = LIST_TAIL_DATA(vrrp_data->vrrp_sync_group);
-	vgroup->script_backup = set_value(strvec);
-	vgroup->notify_exec = 1;
+	vgroup->script_backup = set_vrrp_notify_script(strvec);
+	vgroup->notify_exec = true;
 }
 static void
 vrrp_gnotify_master_handler(vector_t *strvec)
 {
 	vrrp_sgroup_t *vgroup = LIST_TAIL_DATA(vrrp_data->vrrp_sync_group);
-	vgroup->script_master = set_value(strvec);
-	vgroup->notify_exec = 1;
+	vgroup->script_master = set_vrrp_notify_script(strvec);
+	vgroup->notify_exec = true;
 }
 static void
 vrrp_gnotify_fault_handler(vector_t *strvec)
 {
 	vrrp_sgroup_t *vgroup = LIST_TAIL_DATA(vrrp_data->vrrp_sync_group);
-	vgroup->script_fault = set_value(strvec);
-	vgroup->notify_exec = 1;
+	vgroup->script_fault = set_vrrp_notify_script(strvec);
+	vgroup->notify_exec = true;
 }
 static void
 vrrp_gnotify_handler(vector_t *strvec)
 {
 	vrrp_sgroup_t *vgroup = LIST_TAIL_DATA(vrrp_data->vrrp_sync_group);
-	vgroup->script = set_value(strvec);
-	vgroup->notify_exec = 1;
+	vgroup->script = set_vrrp_notify_script(strvec);
+	vgroup->notify_exec = true;
 }
 static void
-vrrp_gsmtp_handler(vector_t *strvec)
+vrrp_gsmtp_handler(__attribute__((unused)) vector_t *strvec)
 {
 	vrrp_sgroup_t *vgroup = LIST_TAIL_DATA(vrrp_data->vrrp_sync_group);
-	vgroup->smtp_alert = 1;
+	vgroup->smtp_alert = true;
 }
 static void
-vrrp_gglobal_tracking_handler(vector_t *strvec)
+vrrp_gglobal_tracking_handler(__attribute__((unused)) vector_t *strvec)
 {
 	vrrp_sgroup_t *vgroup = LIST_TAIL_DATA(vrrp_data->vrrp_sync_group);
 	vgroup->global_tracking = 1;
@@ -165,7 +181,7 @@ vrrp_handler(vector_t *strvec)
 		return;
 	}
 
-	iname = vector_slot(strvec,1);
+	iname = strvec_slot(strvec,1);
 
 	/* Make sure the vrrp instance doesn't already exist */
 	if (!LIST_ISEMPTY(vrrp_data->vrrp)) {
@@ -192,7 +208,7 @@ vrrp_vmac_handler(vector_t *strvec)
 	__set_bit(VRRP_VMAC_BIT, &vrrp->vmac_flags);
 
 	if (vector_size(strvec) >= 2) {
-		strncpy(vrrp->vmac_ifname, vector_slot(strvec, 1), IFNAMSIZ - 1);
+		strncpy(vrrp->vmac_ifname, strvec_slot(strvec, 1), IFNAMSIZ - 1);
 
 		/* Check if the interface exists and is a macvlan we can use */
 		if ((ifp = if_get_by_ifname(vrrp->vmac_ifname)) &&
@@ -203,7 +219,7 @@ vrrp_vmac_handler(vector_t *strvec)
 	}
 }
 static void
-vrrp_vmac_xmit_base_handler(vector_t *strvec)
+vrrp_vmac_xmit_base_handler(__attribute__((unused)) vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 
@@ -211,12 +227,12 @@ vrrp_vmac_xmit_base_handler(vector_t *strvec)
 }
 #endif
 static void
-vrrp_unicast_peer_handler(vector_t *strvec)
+vrrp_unicast_peer_handler(__attribute__((unused)) vector_t *strvec)
 {
-	alloc_value_block(strvec, alloc_vrrp_unicast_peer);
+	alloc_value_block(alloc_vrrp_unicast_peer);
 }
 static void
-vrrp_native_ipv6_handler(vector_t *strvec)
+vrrp_native_ipv6_handler(__attribute__((unused)) vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 
@@ -231,7 +247,7 @@ vrrp_native_ipv6_handler(vector_t *strvec)
 static void
 vrrp_state_handler(vector_t *strvec)
 {
-	char *str = vector_slot(strvec, 1);
+	char *str = strvec_slot(strvec, 1);
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	vrrp_sgroup_t *vgroup = vrrp->sync;
 
@@ -250,7 +266,7 @@ static void
 vrrp_int_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	char *name = vector_slot(strvec, 1);
+	char *name = strvec_slot(strvec, 1);
 
 	vrrp->ifp = if_get_by_ifname(name);
 	if (!vrrp->ifp) {
@@ -260,17 +276,17 @@ vrrp_int_handler(vector_t *strvec)
 	}
 }
 static void
-vrrp_track_int_handler(vector_t *strvec)
+vrrp_track_int_handler(__attribute__((unused)) vector_t *strvec)
 {
-	alloc_value_block(strvec, alloc_vrrp_track);
+	alloc_value_block(alloc_vrrp_track);
 }
 static void
-vrrp_track_scr_handler(vector_t *strvec)
+vrrp_track_scr_handler(__attribute__((unused)) vector_t *strvec)
 {
-	alloc_value_block(strvec, alloc_vrrp_track_script);
+	alloc_value_block(alloc_vrrp_track_script);
 }
 static void
-vrrp_dont_track_handler(vector_t *strvec)
+vrrp_dont_track_handler(__attribute__((unused)) vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	vrrp->dont_track_primary = 1;
@@ -282,7 +298,7 @@ vrrp_srcip_handler(vector_t *strvec)
 	struct sockaddr_storage *saddr = &vrrp->saddr;
 	int ret;
 
-	ret = inet_stosockaddr(vector_slot(strvec, 1), 0, saddr);
+	ret = inet_stosockaddr(strvec_slot(strvec, 1), 0, saddr);
 	if (ret < 0) {
 		log_message(LOG_ERR, "Configuration error: VRRP instance[%s] malformed unicast"
 				     " src address[%s]. Skipping..."
@@ -303,50 +319,51 @@ static void
 vrrp_vrid_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	vrrp->vrid = atoi(vector_slot(strvec, 1));
+	char *end_ptr;
+	unsigned long vrid = strtoul(strvec_slot(strvec, 1),&end_ptr, 10);
 
-	if (VRRP_IS_BAD_VID(vrrp->vrid)) {
-		log_message(LOG_INFO, "VRRP Error : VRID not valid !");
-		log_message(LOG_INFO,
-		       "             must be between 1 & 255. reconfigure !");
-
-		vrrp->vrid = 0;
+	if (*end_ptr || VRRP_IS_BAD_VID(vrid)) {
+		log_message(LOG_INFO, "VRRP Error : VRID not valid - must be between 1 & 255. reconfigure !");
 		return;
 	}
 
+	vrrp->vrid = (uint8_t)vrid;
 	alloc_vrrp_bucket(vrrp);
 }
 static void
 vrrp_prio_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	vrrp->base_priority = atoi(vector_slot(strvec, 1));
+	char *endptr;
+	unsigned long base_priority = strtoul(strvec_slot(strvec, 1), &endptr, 10);
 
-	if (VRRP_IS_BAD_PRIORITY(vrrp->base_priority)) {
+	if (*endptr || VRRP_IS_BAD_PRIORITY(base_priority)) {
 		log_message(LOG_INFO, "(%s): Priority not valid! must be between 1 & 255. Reconfigure !", vrrp->iname);
 		log_message(LOG_INFO, "%*sUsing default value : %d", (int)strlen(vrrp->iname) + 4, "", VRRP_PRIO_DFL);
 
 		vrrp->base_priority = VRRP_PRIO_DFL;
 	}
+	else
+		vrrp->base_priority = (uint8_t)base_priority;
 	vrrp->effective_priority = vrrp->base_priority;
 }
 static void
 vrrp_adv_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	int adver_int = atof(vector_slot(strvec, 1)) * TIMER_HZ;
+	int adver_int = (int)(atof(strvec_slot(strvec, 1)) * TIMER_HZ);
 
 	/* Simple check - just positive */
 	if (adver_int <= 0)
 		log_message(LOG_INFO, "(%s): Advert interval (%s) not valid! Must be > 0 - ignoring", vrrp->iname, FMT_STR_VSLOT(strvec, 1));
 	else
-		vrrp->adver_int = adver_int;
+		vrrp->adver_int = (unsigned)adver_int;
 }
 static void
 vrrp_debug_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	vrrp->debug = atoi(vector_slot(strvec, 1));
+	vrrp->debug = atoi(strvec_slot(strvec, 1));
 
 	if (VRRP_IS_BAD_DEBUG_INT(vrrp->debug)) {
 		log_message(LOG_INFO, "(%s): Debug value not valid! must be between 0-4", vrrp->iname);
@@ -360,9 +377,9 @@ vrrp_skip_check_adv_addr_handler(vector_t *strvec)
 	int res;
 
 	if (vector_size(strvec) >= 2) {
-		res = check_true_false(vector_slot(strvec, 1));
+		res = check_true_false(strvec_slot(strvec, 1));
 		if (res >= 0)
-			vrrp->skip_check_adv_addr = res;
+			vrrp->skip_check_adv_addr = (bool)res;
 		else
 			log_message(LOG_INFO, "(%s): invalid skip_check_adv_addr %s specified", vrrp->iname, FMT_STR_VSLOT(strvec, 1));
 	} else {
@@ -377,9 +394,9 @@ vrrp_strict_mode_handler(vector_t *strvec)
 
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	if (vector_size(strvec) >= 2) {
-		res = check_true_false(vector_slot(strvec, 1));
+		res = check_true_false(strvec_slot(strvec, 1));
 		if (res >= 0)
-			vrrp->strict_mode = res;
+			vrrp->strict_mode = (bool)res;
 		else
 			log_message(LOG_INFO, "(%s): invalid strict_mode %s specified", vrrp->iname, FMT_STR_VSLOT(strvec, 1));
 	} else {
@@ -388,13 +405,13 @@ vrrp_strict_mode_handler(vector_t *strvec)
 	}
 }
 static void
-vrrp_nopreempt_handler(vector_t *strvec)
+vrrp_nopreempt_handler(__attribute__((unused)) vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	vrrp->nopreempt = 1;
 }
 static void	/* backwards compatibility */
-vrrp_preempt_handler(vector_t *strvec)
+vrrp_preempt_handler(__attribute__((unused)) vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	vrrp->nopreempt = 0;
@@ -403,55 +420,56 @@ static void
 vrrp_preempt_delay_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	vrrp->preempt_delay = atoi(vector_slot(strvec, 1));
+	unsigned long preempt_delay = strtoul(strvec_slot(strvec, 1), NULL, 10);
 
-	if (VRRP_IS_BAD_PREEMPT_DELAY(vrrp->preempt_delay)) {
+	if (VRRP_IS_BAD_PREEMPT_DELAY(preempt_delay)) {
 		log_message(LOG_INFO, "(%s): Preempt_delay not valid! must be between 0-%d", vrrp->iname, TIMER_MAX_SEC);
 		vrrp->preempt_delay = 0;
 	}
-	vrrp->preempt_delay *= TIMER_HZ;
+	else
+		vrrp->preempt_delay = preempt_delay * TIMER_HZ;
 	vrrp->preempt_time = timer_add_long(timer_now(), vrrp->preempt_delay);
 }
 static void
 vrrp_notify_backup_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	vrrp->script_backup = set_value(strvec);
-	vrrp->notify_exec = 1;
+	vrrp->script_backup = set_vrrp_notify_script(strvec);
+	vrrp->notify_exec = true;
 }
 static void
 vrrp_notify_master_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	vrrp->script_master = set_value(strvec);
-	vrrp->notify_exec = 1;
+	vrrp->script_master = set_vrrp_notify_script(strvec);
+	vrrp->notify_exec = true;
 }
 static void
 vrrp_notify_fault_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	vrrp->script_fault = set_value(strvec);
-	vrrp->notify_exec = 1;
+	vrrp->script_fault = set_vrrp_notify_script(strvec);
+	vrrp->notify_exec = true;
 }
 static void
 vrrp_notify_stop_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	vrrp->script_stop = set_value(strvec);
-	vrrp->notify_exec = 1;
+	vrrp->script_stop = set_vrrp_notify_script(strvec);
+	vrrp->notify_exec = true;
 }
 static void
 vrrp_notify_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	vrrp->script = set_value(strvec);
-	vrrp->notify_exec = 1;
+	vrrp->script = set_vrrp_notify_script(strvec);
+	vrrp->notify_exec = true;
 }
 static void
-vrrp_smtp_handler(vector_t *strvec)
+vrrp_smtp_handler(__attribute__((unused)) vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	vrrp->smtp_alert = 1;
+	vrrp->smtp_alert = true;
 }
 #ifdef _WITH_LVS_
 static void
@@ -475,7 +493,7 @@ static void
 vrrp_garp_delay_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	vrrp->garp_delay = atoi(vector_slot(strvec, 1)) * TIMER_HZ;
+	vrrp->garp_delay = (unsigned)strtoul(strvec_slot(strvec, 1), NULL, 10) * TIMER_HZ;
 	if (vrrp->garp_delay < TIMER_HZ)
 		vrrp->garp_delay = TIMER_HZ;
 }
@@ -483,14 +501,14 @@ static void
 vrrp_garp_refresh_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	vrrp->garp_refresh.tv_sec = atoi(vector_slot(strvec, 1));
+	vrrp->garp_refresh.tv_sec = atoi(strvec_slot(strvec, 1));
 	vrrp->garp_refresh.tv_usec = 0;
 }
 static void
 vrrp_garp_rep_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	vrrp->garp_rep = atoi(vector_slot(strvec, 1));
+	vrrp->garp_rep = (unsigned)strtoul(strvec_slot(strvec, 1), NULL, 10);
 	if (vrrp->garp_rep < 1)
 		vrrp->garp_rep = 1;
 }
@@ -498,7 +516,7 @@ static void
 vrrp_garp_refresh_rep_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	vrrp->garp_refresh_rep = atoi(vector_slot(strvec, 1));
+	vrrp->garp_refresh_rep = (unsigned)strtoul(strvec_slot(strvec, 1), NULL, 10);
 	if (vrrp->garp_refresh_rep < 1)
 		vrrp->garp_refresh_rep = 1;
 }
@@ -507,16 +525,19 @@ static void
 vrrp_garp_lower_prio_delay_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	vrrp->garp_lower_prio_delay = atoi(vector_slot(strvec, 1)) * TIMER_HZ;
+	vrrp->garp_lower_prio_delay = (unsigned)strtoul(strvec_slot(strvec, 1), NULL, 10) * TIMER_HZ;
 }
 static void
 vrrp_garp_lower_prio_rep_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	vrrp->garp_lower_prio_rep = atoi(vector_slot(strvec, 1));
+	int garp_lower_prio_rep = atoi(strvec_slot(strvec, 1));
+
 	/* Allow 0 GARP messages to be sent */
-	if ( vrrp->garp_lower_prio_rep < 0 )
+	if (garp_lower_prio_rep < 0)
 		vrrp->garp_lower_prio_rep = 0;
+	else
+		vrrp->garp_lower_prio_rep = (unsigned)garp_lower_prio_rep;
 }
 static void
 vrrp_lower_prio_no_advert_handler(vector_t *strvec)
@@ -525,9 +546,9 @@ vrrp_lower_prio_no_advert_handler(vector_t *strvec)
 
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	if (vector_size(strvec) >= 2) {
-		res = check_true_false(vector_slot(strvec, 1));
+		res = check_true_false(strvec_slot(strvec, 1));
 		if (res >= 0)
-			vrrp->lower_prio_no_advert = res;
+			vrrp->lower_prio_no_advert = (unsigned)res;
 		else
 			log_message(LOG_INFO, "(%s): invalid lower_prio_no_advert %s specified", vrrp->iname, FMT_STR_VSLOT(strvec, 1));
 	} else {
@@ -542,7 +563,7 @@ static void
 vrrp_auth_type_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	char *str = vector_slot(strvec, 1);
+	char *str = strvec_slot(strvec, 1);
 
 	if (!strcmp(str, "AH"))
 		vrrp->auth_type = VRRP_AUTH_AH;
@@ -555,14 +576,14 @@ static void
 vrrp_auth_pass_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
-	char *str = vector_slot(strvec, 1);
-	int max_size = sizeof (vrrp->auth_data);
-	int str_len = strlen(str);
+	char *str = strvec_slot(strvec, 1);
+	size_t max_size = sizeof (vrrp->auth_data);
+	size_t str_len = strlen(str);
 
 	if (str_len > max_size) {
 		str_len = max_size;
 		log_message(LOG_INFO,
-			    "Truncating auth_pass to %d characters", max_size);
+			    "Truncating auth_pass to %zu characters", max_size);
 	}
 
 	memset(vrrp->auth_data, 0, max_size);
@@ -570,20 +591,20 @@ vrrp_auth_pass_handler(vector_t *strvec)
 }
 #endif
 static void
-vrrp_vip_handler(vector_t *strvec)
+vrrp_vip_handler(__attribute__((unused)) vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 	char *buf;
 	char *str = NULL;
 	vector_t *vec = NULL;
-	int address_family;
+	sa_family_t address_family;
 
 	buf = (char *) MALLOC(MAXBUF);
 	while (read_line(buf, MAXBUF)) {
 		address_family = AF_UNSPEC;
 		vec = alloc_strvec(buf);
 		if (vec) {
-			str = vector_slot(vec, 0);
+			str = strvec_slot(vec, 0);
 			if (!strcmp(str, EOB)) {
 				free_strvec(vec);
 				break;
@@ -611,38 +632,47 @@ vrrp_vip_handler(vector_t *strvec)
 	FREE(buf);
 }
 static void
-vrrp_evip_handler(vector_t *strvec)
+vrrp_evip_handler(__attribute__((unused)) vector_t *strvec)
 {
-	alloc_value_block(strvec, alloc_vrrp_evip);
+	alloc_value_block(alloc_vrrp_evip);
+}
+static void
+vrrp_promote_secondaries(__attribute__((unused)) vector_t *strvec)
+{
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+
+	vrrp->promote_secondaries = true;
 }
 #ifdef _HAVE_FIB_ROUTING_
 static void
-vrrp_vroutes_handler(vector_t *strvec)
+vrrp_vroutes_handler(__attribute__((unused)) vector_t *strvec)
 {
-	alloc_value_block(strvec, alloc_vrrp_vroute);
+	alloc_value_block(alloc_vrrp_vroute);
 }
 static void
-vrrp_vrules_handler(vector_t *strvec)
+vrrp_vrules_handler(__attribute__((unused)) vector_t *strvec)
 {
-	alloc_value_block(strvec, alloc_vrrp_vrule);
+	alloc_value_block(alloc_vrrp_vrule);
 }
 #endif
 static void
 vrrp_script_handler(vector_t *strvec)
 {
-	alloc_vrrp_script(vector_slot(strvec, 1));
+	alloc_vrrp_script(strvec_slot(strvec, 1));
 }
 static void
 vrrp_vscript_script_handler(vector_t *strvec)
 {
 	vrrp_script_t *vscript = LIST_TAIL_DATA(vrrp_data->vrrp_script);
 	vscript->script = set_value(strvec);
+	vscript->uid = default_script_uid;
+	vscript->gid = default_script_gid;
 }
 static void
 vrrp_vscript_interval_handler(vector_t *strvec)
 {
 	vrrp_script_t *vscript = LIST_TAIL_DATA(vrrp_data->vrrp_script);
-	vscript->interval = atoi(vector_slot(strvec, 1)) * TIMER_HZ;
+	vscript->interval = (unsigned)(strtoul(strvec_slot(strvec, 1), NULL, 10) * TIMER_HZ);
 	if (vscript->interval < TIMER_HZ)
 		vscript->interval = TIMER_HZ;
 }
@@ -650,7 +680,7 @@ static void
 vrrp_vscript_timeout_handler(vector_t *strvec)
 {
 	vrrp_script_t *vscript = LIST_TAIL_DATA(vrrp_data->vrrp_script);
-	vscript->timeout = atoi(vector_slot(strvec, 1)) * TIMER_HZ;
+	vscript->timeout = strtoul(strvec_slot(strvec, 1), NULL, 10) * TIMER_HZ;
 	if (vscript->timeout < TIMER_HZ)
 		vscript->timeout = TIMER_HZ;
 }
@@ -658,13 +688,13 @@ static void
 vrrp_vscript_weight_handler(vector_t *strvec)
 {
 	vrrp_script_t *vscript = LIST_TAIL_DATA(vrrp_data->vrrp_script);
-	vscript->weight = atoi(vector_slot(strvec, 1));
+	vscript->weight = atoi(strvec_slot(strvec, 1));
 }
 static void
 vrrp_vscript_rise_handler(vector_t *strvec)
 {
 	vrrp_script_t *vscript = LIST_TAIL_DATA(vrrp_data->vrrp_script);
-	vscript->rise = atoi(vector_slot(strvec, 1));
+	vscript->rise = atoi(strvec_slot(strvec, 1));
 	if (vscript->rise < 1)
 		vscript->rise = 1;
 }
@@ -672,17 +702,29 @@ static void
 vrrp_vscript_fall_handler(vector_t *strvec)
 {
 	vrrp_script_t *vscript = LIST_TAIL_DATA(vrrp_data->vrrp_script);
-	vscript->fall = atoi(vector_slot(strvec, 1));
+	vscript->fall = atoi(strvec_slot(strvec, 1));
 	if (vscript->fall < 1)
 		vscript->fall = 1;
 }
-
+static void
+vrrp_vscript_user_handler(vector_t *strvec)
+{
+	vrrp_script_t *vscript = LIST_TAIL_DATA(vrrp_data->vrrp_script);
+	if (set_script_uid_gid(strvec, 1, &vscript->uid, &vscript->gid))
+		log_message(LOG_INFO, "Unable to set uid/gid for script %s", vscript->script);
+}
+static void
+vrrp_vscript_init_fail_handler(__attribute__((unused)) vector_t *strvec)
+{
+	vrrp_script_t *vscript = LIST_TAIL_DATA(vrrp_data->vrrp_script);
+	vscript->result = VRRP_SCRIPT_STATUS_INIT_FAILED;
+}
 static void
 vrrp_version_handler(vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+	uint8_t version = (uint8_t)strtoul(strvec_slot(strvec, 1), NULL, 10);
 
-	uint8_t version = atoi(vector_slot(strvec, 1));
 	if (VRRP_IS_BAD_VERSION(version)) {
 		log_message(LOG_INFO, "VRRP Error : Version not valid !");
 		log_message(LOG_INFO, "             must be between either 2 or 3. reconfigure !");
@@ -699,7 +741,7 @@ vrrp_version_handler(vector_t *strvec)
 }
 
 static void
-vrrp_accept_handler(vector_t *strvec)
+vrrp_accept_handler(__attribute__((unused)) vector_t *strvec)
 {
 	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
 
@@ -707,7 +749,15 @@ vrrp_accept_handler(vector_t *strvec)
 }
 
 static void
-garp_group_handler(vector_t *strvec)
+vrrp_no_accept_handler(__attribute__((unused)) vector_t *strvec)
+{
+	vrrp_t *vrrp = LIST_TAIL_DATA(vrrp_data->vrrp);
+
+	vrrp->accept = false;
+}
+
+static void
+garp_group_handler(__attribute__((unused)) vector_t *strvec)
 {
 	alloc_garp_delay();
 }
@@ -716,7 +766,7 @@ garp_group_garp_interval_handler(vector_t *strvec)
 {
 	garp_delay_t *delay = LIST_TAIL_DATA(garp_delay);
 
-	delay->garp_interval.tv_usec = atof(vector_slot(strvec, 1)) * 1000000;
+	delay->garp_interval.tv_usec = (useconds_t)atof(strvec_slot(strvec, 1)) * 1000000;
 	delay->garp_interval.tv_sec = delay->garp_interval.tv_usec / 1000000;
 	delay->garp_interval.tv_usec %= 1000000;
 	delay->have_garp_interval = true;
@@ -729,7 +779,7 @@ garp_group_gna_interval_handler(vector_t *strvec)
 {
 	garp_delay_t *delay = LIST_TAIL_DATA(garp_delay);
 
-	delay->gna_interval.tv_usec = atof(vector_slot(strvec, 1)) * 1000000;
+	delay->gna_interval.tv_usec = (suseconds_t)(atof(strvec_slot(strvec, 1)) * 1000000);
 	delay->gna_interval.tv_sec = delay->gna_interval.tv_usec / 1000000;
 	delay->gna_interval.tv_usec %= 1000000;
 	delay->have_gna_interval = true;
@@ -740,7 +790,7 @@ garp_group_gna_interval_handler(vector_t *strvec)
 static void
 garp_group_interface_handler(vector_t *strvec)
 {
-	interface_t *ifp = if_get_by_ifname(vector_slot(strvec, 1));
+	interface_t *ifp = if_get_by_ifname(strvec_slot(strvec, 1));
 	if (!ifp) {
 		log_message(LOG_INFO, "Unknown interface %s specified for garp_group - ignoring", FMT_STR_VSLOT(strvec, 1));
 		return;
@@ -764,8 +814,8 @@ garp_group_interfaces_handler(vector_t *strvec)
 {
 	garp_delay_t *delay = LIST_TAIL_DATA(garp_delay);
 	interface_t *ifp;
-        vector_t *interface_vec = read_value_block(strvec);
-	int i;
+	vector_t *interface_vec = read_value_block(strvec);
+	size_t i;
 	garp_delay_t *gd;
 	element e;
 
@@ -777,7 +827,7 @@ garp_group_interfaces_handler(vector_t *strvec)
 			delay->aggregation_group++;
 	}
 
-        for (i = 0; i < vector_size(interface_vec); i++) {
+	for (i = 0; i < vector_size(interface_vec); i++) {
 		ifp = if_get_by_ifname(vector_slot(interface_vec, i));
 		if (!ifp) {
 			log_message(LOG_INFO, "Unknown interface %s specified for garp_group - ignoring", FMT_STR_VSLOT(interface_vec, i));
@@ -798,7 +848,7 @@ garp_group_interfaces_handler(vector_t *strvec)
 		ifp->garp_delay = delay;
 	}
 
-        free_strvec(interface_vec);
+	free_strvec(interface_vec);
 }
 
 void
@@ -847,11 +897,13 @@ init_vrrp_keywords(bool active)
 	install_keyword("advert_int", &vrrp_adv_handler);
 	install_keyword("virtual_ipaddress", &vrrp_vip_handler);
 	install_keyword("virtual_ipaddress_excluded", &vrrp_evip_handler);
+	install_keyword("promote_secondaries", &vrrp_promote_secondaries);
 #ifdef _HAVE_FIB_ROUTING_
 	install_keyword("virtual_routes", &vrrp_vroutes_handler);
 	install_keyword("virtual_rules", &vrrp_vrules_handler);
 #endif
 	install_keyword("accept", &vrrp_accept_handler);
+	install_keyword("no_accept", &vrrp_no_accept_handler);
 	install_keyword("skip_check_adv_addr", &vrrp_skip_check_adv_addr_handler);
 	install_keyword("strict_mode", &vrrp_strict_mode_handler);
 	install_keyword("preempt", &vrrp_preempt_handler);
@@ -888,6 +940,8 @@ init_vrrp_keywords(bool active)
 	install_keyword("weight", &vrrp_vscript_weight_handler);
 	install_keyword("rise", &vrrp_vscript_rise_handler);
 	install_keyword("fall", &vrrp_vscript_fall_handler);
+	install_keyword("user", &vrrp_vscript_user_handler);
+	install_keyword("init_fail", &vrrp_vscript_init_fail_handler);
 }
 
 vector_t *
