@@ -62,17 +62,17 @@ vgroup_print(FILE *file, void *data)
 		fprintf(file, "   monitor = %s\n", str);
 	}
 	if (vgroup->script_backup)
-		fprintf(file, "   Backup state transition script = %s\n",
-		       vgroup->script_backup);
+		fprintf(file, "   Backup state transition script = %s, uid:gid %d:%d\n",
+		       vgroup->script_backup->name, vgroup->script_backup->uid, vgroup->script_backup->gid);
 	if (vgroup->script_master)
-		fprintf(file, "   Master state transition script = %s\n",
-		       vgroup->script_master);
+		fprintf(file, "   Master state transition script = %s, uid:gid %d:%d\n",
+		       vgroup->script_master->name, vgroup->script_master->uid, vgroup->script_master->gid);
 	if (vgroup->script_fault)
-		fprintf(file, "   Fault state transition script = %s\n",
-		       vgroup->script_fault);
+		fprintf(file, "   Fault state transition script = %s, uid:gid %d:%d\n",
+		       vgroup->script_fault->name, vgroup->script_fault->uid, vgroup->script_fault->gid);
 	if (vgroup->script)
-		fprintf(file, "   Generic state transition script = '%s\n'",
-		       vgroup->script);
+		fprintf(file, "   Generic state transition script = '%s', uid:gid %d:%d\n",
+		       vgroup->script->name, vgroup->script->uid, vgroup->script->gid);
 	if (vgroup->smtp_alert)
 		fprintf(file, "   Using smtp notification\n");
 
@@ -87,16 +87,19 @@ vscript_print(FILE *file, void *data)
 
 	fprintf(file, " VRRP Script = %s\n", vscript->sname);
 	fprintf(file, "   Command = %s\n", vscript->script);
-	fprintf(file, "   Interval = %ld sec\n", vscript->interval / TIMER_HZ);
+	fprintf(file, "   Interval = %lu sec\n", vscript->interval / TIMER_HZ);
 	fprintf(file, "   Weight = %d\n", vscript->weight);
 	fprintf(file, "   Rise = %d\n", vscript->rise);
 	fprintf(file, "   Full = %d\n", vscript->fall);
+	fprintf(file, "   Insecure = %s\n", vscript->insecure ? "yes" : "no");
 
 	switch (vscript->result) {
 	case VRRP_SCRIPT_STATUS_INIT:
 		str = "INIT"; break;
 	case VRRP_SCRIPT_STATUS_INIT_GOOD:
 		str = "INIT/GOOD"; break;
+	case VRRP_SCRIPT_STATUS_INIT_FAILED:
+		str = "INIT/FAILED"; break;
 	case VRRP_SCRIPT_STATUS_DISABLED:
 		str = "DISABLED"; break;
 	default:
@@ -172,7 +175,7 @@ if_print(FILE *file, void * data)
 
 	fprintf(file, "------< NIC >------\n");
 	fprintf(file, " Name = %s\n", ifp->ifname);
-	fprintf(file, " index = %d\n", ifp->ifindex);
+	fprintf(file, " index = %u\n", ifp->ifindex);
 	fprintf(file, " IPv4 address = %s\n",
 		inet_ntop2(ifp->sin_addr.s_addr));
 	inet_ntop(AF_INET6, &ifp->sin6_addr, addr_str, sizeof(addr_str));
@@ -263,8 +266,8 @@ vrrp_print(FILE *file, void *data)
 	fprintf(file, "   Gratuitous ARP refresh = %lu\n",
 		       vrrp->garp_refresh.tv_sec/TIMER_HZ);
 	fprintf(file, "   Gratuitous ARP refresh repeat = %d\n", vrrp->garp_refresh_rep);
-	fprintf(file, "   Gratuitous ARP lower priority delay = %d\n", vrrp->garp_lower_prio_delay / TIMER_HZ);
-	fprintf(file, "   Gratuitous ARP lower priority repeat = %d\n", vrrp->garp_lower_prio_rep);
+	fprintf(file, "   Gratuitous ARP lower priority delay = %u\n", vrrp->garp_lower_prio_delay / TIMER_HZ);
+	fprintf(file, "   Gratuitous ARP lower priority repeat = %u\n", vrrp->garp_lower_prio_rep);
 	fprintf(file, "   Send advert after receive lower priority advert = %s\n", vrrp->lower_prio_no_advert ? "false" : "true");
 	fprintf(file, "   Virtual Router ID = %d\n", vrrp->vrid);
 	fprintf(file, "   Priority = %d\n", vrrp->base_priority);
@@ -272,11 +275,9 @@ vrrp_print(FILE *file, void *data)
 		(vrrp->version == VRRP_VERSION_2) ? (vrrp->adver_int / TIMER_HZ) :
 		(vrrp->adver_int / (TIMER_HZ / 1000)),
 		(vrrp->version == VRRP_VERSION_2) ? "sec" : "milli-sec");
-	fprintf(file, "   Accept = %s\n", ((vrrp->accept) ? "enabled" : "disabled"));
-	if (vrrp->nopreempt)
-		fprintf(file, "   Preempt = disabled\n");
-	else
-		fprintf(file, "   Preempt = enabled\n");
+	fprintf(file, "   Accept = %s\n", vrrp->accept ? "enabled" : "disabled");
+	fprintf(file, "   Preempt = %s\n", vrrp->nopreempt ? "disabled" : "enabled");
+	fprintf(file, "   Promote_secondaries = %s\n", vrrp->promote_secondaries ? "enabled" : "disabled");
 	if (vrrp->preempt_delay)
 		fprintf(file, "   Preempt delay = %ld secs\n",
 		       vrrp->preempt_delay / TIMER_HZ);
@@ -326,20 +327,20 @@ vrrp_print(FILE *file, void *data)
 	}
 #endif
 	if (vrrp->script_backup)
-		fprintf(file, "   Backup state transition script = %s\n",
-		       vrrp->script_backup);
+		fprintf(file, "   Backup state transition script = %s, uid:gid %d:%d\n",
+				vrrp->script_backup->name, vrrp->script_backup->uid, vrrp->script_backup->gid);
 	if (vrrp->script_master)
-		fprintf(file, "   Master state transition script = %s\n",
-		       vrrp->script_master);
+		fprintf(file, "   Master state transition script = %s, uid:gid %d:%d\n",
+				vrrp->script_master->name, vrrp->script_master->uid, vrrp->script_master->gid);
 	if (vrrp->script_fault)
-		fprintf(file, "   Fault state transition script = %s\n",
-		       vrrp->script_fault);
+		fprintf(file, "   Fault state transition script = %s, uid:gid %d:%d\n",
+				vrrp->script_fault->name, vrrp->script_fault->uid, vrrp->script_fault->gid);
 	if (vrrp->script_stop)
-		fprintf(file, "   Stop state transition script = %s\n",
-		       vrrp->script_stop);
+		fprintf(file, "   Stop state transition script = %s, uid:gid %d:%d\n",
+				vrrp->script_stop->name, vrrp->script_stop->uid, vrrp->script_stop->gid);
 	if (vrrp->script)
-		fprintf(file, "   Generic state transition script = '%s'\n",
-		       vrrp->script);
+		fprintf(file, "   Generic state transition script = '%s', uid:gid %d:%d\n",
+				vrrp->script->name, vrrp->script->uid, vrrp->script->gid);
 	if (vrrp->smtp_alert)
 			fprintf(file, "   Using smtp notification\n");
 }
