@@ -73,7 +73,7 @@ netlink_link_up(vrrp_t *vrrp)
 	req.n.nlmsg_flags = NLM_F_REQUEST;
 	req.n.nlmsg_type = RTM_NEWLINK;
 	req.ifi.ifi_family = AF_UNSPEC;
-	req.ifi.ifi_index = IF_INDEX(vrrp->ifp);
+	req.ifi.ifi_index = (int)IF_INDEX(vrrp->ifp);
 	req.ifi.ifi_change |= IFF_UP;
 	req.ifi.ifi_flags |= IFF_UP;
 
@@ -88,10 +88,11 @@ netlink_link_add_vmac(vrrp_t *vrrp)
 {
 	struct rtattr *linkinfo;
 	struct rtattr *data;
-	unsigned int base_ifindex;
+	ifindex_t base_ifindex;
 	interface_t *ifp;
 	interface_t *base_ifp;
 	char ifname[IFNAMSIZ];
+	bool create_interface = true;
 	struct {
 		struct nlmsghdr n;
 		struct ifinfomsg ifi;
@@ -118,73 +119,82 @@ netlink_link_add_vmac(vrrp_t *vrrp)
 	 */
 	if ((ifp = if_get_by_ifname(ifname))) {
 		/* Check to see whether this interface has wrong mac ? */
-		if (memcmp((const void *) ifp->hw_addr,
-			   (const void *) ll_addr, ETH_ALEN) != 0) {
-			/* We have found a VIF but the vmac do not match */
-			log_message(LOG_INFO, "vmac: Removing old VMAC interface %s due to conflicting "
-					      "interface MAC for vrrp_instance %s!!!"
-					    , vrrp->vmac_ifname, vrrp->iname);
+		if ((memcmp((const void *) ifp->hw_addr, (const void *) ll_addr, ETH_ALEN) != 0 ||
+		     ifp->base_ifindex != vrrp->ifp->ifindex)) {
 
-			/* Request that NETLINK remove the VIF interface first */
-			memset(&req, 0, sizeof (req));
-			req.n.nlmsg_len = NLMSG_LENGTH(sizeof (struct ifinfomsg));
-			req.n.nlmsg_flags = NLM_F_REQUEST;
-			req.n.nlmsg_type = RTM_DELLINK;
-			req.ifi.ifi_family = AF_INET;
-			req.ifi.ifi_index = IF_INDEX(ifp);
-
-			if (netlink_talk(&nl_cmd, &req.n) < 0) {
-				log_message(LOG_INFO, "vmac: Error removing VMAC interface %s for "
-						      "vrrp_instance %s!!!"
+			/* Be safe here - we don't want to remove a physical interface */
+			if (ifp->vmac) {
+				/* We have found a VIF but the vmac do not match */
+				log_message(LOG_INFO, "vmac: Removing old VMAC interface %s due to conflicting "
+						      "interface or MAC for vrrp_instance %s!!!"
 						    , vrrp->vmac_ifname, vrrp->iname);
-				return -1;
+
+				/* Request that NETLINK remove the VIF interface first */
+				memset(&req, 0, sizeof (req));
+				req.n.nlmsg_len = NLMSG_LENGTH(sizeof (struct ifinfomsg));
+				req.n.nlmsg_flags = NLM_F_REQUEST;
+				req.n.nlmsg_type = RTM_DELLINK;
+				req.ifi.ifi_family = AF_INET;
+				req.ifi.ifi_index = (int)IF_INDEX(ifp);
+
+				if (netlink_talk(&nl_cmd, &req.n) < 0) {
+					log_message(LOG_INFO, "vmac: Error removing VMAC interface %s for "
+							      "vrrp_instance %s!!!"
+							    , vrrp->vmac_ifname, vrrp->iname);
+					return -1;
+				}
+
+				/* Interface successfully removed, now recreate */
+				ifp = NULL;
 			}
-
-			/* Interface successfully removed, now recreate */
 		}
+		else
+			create_interface = false;
 	}
 
-	/* Request that NETLINK create the VIF interface */
-	req.n.nlmsg_len = NLMSG_LENGTH(sizeof (struct ifinfomsg));
-	req.n.nlmsg_flags = NLM_F_REQUEST | NLM_F_CREATE | NLM_F_EXCL;
-	req.n.nlmsg_type = RTM_NEWLINK;
-	req.ifi.ifi_family = AF_INET;
+	if (create_interface) {
+		/* Request that NETLINK create the VIF interface */
+		req.n.nlmsg_len = NLMSG_LENGTH(sizeof (struct ifinfomsg));
+		req.n.nlmsg_flags = NLM_F_REQUEST | NLM_F_CREATE | NLM_F_EXCL;
+		req.n.nlmsg_type = RTM_NEWLINK;
+		req.ifi.ifi_family = AF_INET;
 
-	/* macvlan settings */
-	linkinfo = NLMSG_TAIL(&req.n);
-	addattr_l(&req.n, sizeof(req), IFLA_LINKINFO, NULL, 0);
-	addattr_l(&req.n, sizeof(req), IFLA_INFO_KIND, (void *)macvlan_ll_kind, strlen(macvlan_ll_kind));
-	data = NLMSG_TAIL(&req.n);
-	addattr_l(&req.n, sizeof(req), IFLA_INFO_DATA, NULL, 0);
+		/* macvlan settings */
+		linkinfo = NLMSG_TAIL(&req.n);
+		addattr_l(&req.n, sizeof(req), IFLA_LINKINFO, NULL, 0);
+		addattr_l(&req.n, sizeof(req), IFLA_INFO_KIND, (void *)macvlan_ll_kind, strlen(macvlan_ll_kind));
+		data = NLMSG_TAIL(&req.n);
+		addattr_l(&req.n, sizeof(req), IFLA_INFO_DATA, NULL, 0);
 
-	/*
-	 * In private mode, macvlan will receive frames with same MAC addr
-	 * as configured on the interface.
-	 */
-	addattr32(&req.n, sizeof(req), IFLA_MACVLAN_MODE,
-		  MACVLAN_MODE_PRIVATE);
-	data->rta_len = (void *)NLMSG_TAIL(&req.n) - (void *)data;
-	linkinfo->rta_len = (void *)NLMSG_TAIL(&req.n) - (void *)linkinfo;
-	addattr_l(&req.n, sizeof(req), IFLA_LINK, &IF_INDEX(vrrp->ifp), sizeof(uint32_t));
-	addattr_l(&req.n, sizeof(req), IFLA_IFNAME, ifname, strlen(ifname));
-	addattr_l(&req.n, sizeof(req), IFLA_ADDRESS, ll_addr, ETH_ALEN);
+		/*
+		 * In private mode, macvlan will receive frames with same MAC addr
+		 * as configured on the interface.
+		 */
+		addattr32(&req.n, sizeof(req), IFLA_MACVLAN_MODE, MACVLAN_MODE_PRIVATE);
+		data->rta_len = (unsigned short)((void *)NLMSG_TAIL(&req.n) - (void *)data);
+		linkinfo->rta_len = (unsigned short)((void *)NLMSG_TAIL(&req.n) - (void *)linkinfo);
+		addattr_l(&req.n, sizeof(req), IFLA_LINK, &IF_INDEX(vrrp->ifp), sizeof(uint32_t));
+		addattr_l(&req.n, sizeof(req), IFLA_IFNAME, ifname, strlen(ifname));
+		addattr_l(&req.n, sizeof(req), IFLA_ADDRESS, ll_addr, ETH_ALEN);
 
-	if (netlink_talk(&nl_cmd, &req.n) < 0) {
-		log_message(LOG_INFO, "vmac: Error creating VMAC interface %s for vrrp_instance %s!!!"
+		if (netlink_talk(&nl_cmd, &req.n) < 0) {
+			log_message(LOG_INFO, "vmac: Error creating VMAC interface %s for vrrp_instance %s!!!"
+					    , ifname, vrrp->iname);
+			return -1;
+		}
+
+		log_message(LOG_INFO, "vmac: Success creating VMAC interface %s for vrrp_instance %s"
 				    , ifname, vrrp->iname);
-		return -1;
+
+		/*
+		 * Update interface queue and vrrp instance interface binding.
+		 */
+		netlink_interface_lookup();
+		ifp = if_get_by_ifname(ifname);
+		if (!ifp)
+			return -1;
 	}
 
-	log_message(LOG_INFO, "vmac: Success creating VMAC interface %s for vrrp_instance %s"
-			    , ifname, vrrp->iname);
-
-	/*
-	 * Update interface queue and vrrp instance interface binding.
-	 */
-	netlink_interface_lookup();
-	ifp = if_get_by_ifname(ifname);
-	if (!ifp)
-		return -1;
 	base_ifp = vrrp->ifp;
 	base_ifindex = vrrp->ifp->ifindex;
 	ifp->flags = vrrp->ifp->flags; /* Copy base interface flags */
@@ -207,13 +217,13 @@ netlink_link_add_vmac(vrrp_t *vrrp)
 		// If we have a sufficiently recent kernel, we can stop a link local address
 		// based on the MAC address being automatically assigned. If not, then we have
 		// to delete the generated address after bringing the interface up (see below).
-#ifdef IFLA_INET6_ADDR_GEN_MODE		/* Since Linux 3.17 */
+#if HAVE_DECL_IFLA_INET6_ADDR_GEN_MODE
 		memset(&req, 0, sizeof (req));
 		req.n.nlmsg_len = NLMSG_LENGTH(sizeof (struct ifinfomsg));
 		req.n.nlmsg_flags = NLM_F_REQUEST ;
 		req.n.nlmsg_type = RTM_NEWLINK;
 		req.ifi.ifi_family = AF_UNSPEC;
-		req.ifi.ifi_index = vrrp->vmac_ifindex;
+		req.ifi.ifi_index = (int)vrrp->vmac_ifindex;
 
 		u_char val = IN6_ADDR_GEN_MODE_NONE;
 		struct rtattr* spec;
@@ -223,8 +233,8 @@ netlink_link_add_vmac(vrrp_t *vrrp)
 		data = NLMSG_TAIL(&req.n);
 		addattr_l(&req.n, sizeof(req), AF_INET6, NULL,0);
 		addattr_l(&req.n, sizeof(req), IFLA_INET6_ADDR_GEN_MODE, &val, sizeof(val));
-		data->rta_len = (void *)NLMSG_TAIL(&req.n) - (void *)data;
-		spec->rta_len = (void *)NLMSG_TAIL(&req.n) - (void *)spec;
+		data->rta_len = (unsigned short)((void *)NLMSG_TAIL(&req.n) - (void *)data);
+		spec->rta_len = (unsigned short)((void *)NLMSG_TAIL(&req.n) - (void *)spec);
 
 		if (netlink_talk(&nl_cmd, &req.n) < 0)
 			log_message(LOG_INFO, "vmac: Error setting ADDR_GEN_MODE to NONE");
@@ -266,7 +276,7 @@ netlink_link_add_vmac(vrrp_t *vrrp)
 	__set_bit(VRRP_VMAC_UP_BIT, &vrrp->vmac_flags);
 	netlink_link_up(vrrp);
 
-#ifndef IFLA_INET6_ADDR_GEN_MODE
+#if !HAVE_DECL_IFLA_INET6_ADDR_GEN_MODE
 	if (vrrp->family == AF_INET6 || vrrp->evip_add_ipv6) {
 		/* Delete the automatically created link-local address based on the
 		 * MAC address if we weren't able to configure the interface not to
@@ -323,7 +333,7 @@ netlink_link_del_vmac(vrrp_t *vrrp)
 	req.n.nlmsg_flags = NLM_F_REQUEST;
 	req.n.nlmsg_type = RTM_DELLINK;
 	req.ifi.ifi_family = AF_INET;
-	req.ifi.ifi_index = vrrp->vmac_ifindex;
+	req.ifi.ifi_index = (int)vrrp->vmac_ifindex;
 
 	if (netlink_talk(&nl_cmd, &req.n) < 0) {
 		log_message(LOG_INFO, "vmac: Error removing VMAC interface %s for vrrp_instance %s!!!"
