@@ -179,7 +179,7 @@ signal_ignore(int signo)
 
 /* Handlers intialization */
 void
-signal_handler_init(void)
+signal_handler_init(int remember)
 {
 	sigset_t sset;
 	int sig;
@@ -224,8 +224,10 @@ signal_handler_init(void)
 	sigemptyset(&act.sa_mask);
 	act.sa_flags = 0;
 
-	sigemptyset(&ign_sig);
-	sigemptyset(&dfl_sig);
+	if (remember) {
+		sigemptyset(&ign_sig);
+		sigemptyset(&dfl_sig);
+	}
 
 	for (sig = 1; sig <= SIGRTMAX; sig++) {
 		if (sigismember(&sset, sig)){
@@ -234,11 +236,14 @@ signal_handler_init(void)
 			/* Remember the original disposition, and ignore
 			 * any default action signals
 			 */
-			if (oact.sa_handler == SIG_IGN)
-				sigaddset(&ign_sig, sig);
+			if (oact.sa_handler == SIG_IGN) {
+				if (remember)
+					sigaddset(&ign_sig, sig);
+			}
 			else {
 				sigaction(sig, &act, NULL);
-				sigaddset(&dfl_sig, sig);
+				if (remember)
+					sigaddset(&dfl_sig, sig);
 			}
 		}
 	}
