@@ -27,7 +27,7 @@
 /* Prototypes */
 extern void *signal_set(int signo, void (*func) (void *, int), void *);
 extern void *signal_ignore(int signo);
-extern void signal_handler_init(void);
+extern void signal_handler_init(int);
 extern void signal_handler_destroy(void);
 extern void signal_handler_script(void);
 extern void signal_run_callback(void);
