@@ -49,13 +49,13 @@
 extern unsigned long debug;
 
 /* Prototypes defs */
-extern void dump_buffer(char *, int, FILE *);
+extern void dump_buffer(char *, size_t, FILE *);
 #ifdef _WITH_STACKTRACE_
 extern void write_stacktrace(const char *);
 #endif
-extern u_short in_csum(u_short *, int, int, int *);
+extern uint16_t in_csum(const uint16_t *, size_t, uint32_t, uint32_t *);
 extern char *inet_ntop2(uint32_t);
-extern uint8_t inet_stor(const char *);
+extern uint32_t inet_stor(const char *);
 extern int domain_stosockaddr(const char *, const char *, struct sockaddr_storage *);
 extern int inet_stosockaddr(char *, const char *, struct sockaddr_storage *);
 extern void inet_ip4tosockaddr(struct in_addr *, struct sockaddr_storage *);
